from __future__ import annotations

from typing import Any, Callable

from Live import *
from Live.Application import *
from Live.Base import *
from Live.Browser import *
from Live.CcControlDevice import *
from Live.Chain import *
from Live.ChainMixerDevice import *
from Live.Clip import *
from Live.ClipSlot import *
from Live.CompressorDevice import *
from Live.Conversions import *
from Live.Device import *
from Live.DeviceIO import *
from Live.DeviceParameter import *
from Live.DriftDevice import *
from Live.DrumChain import *
from Live.DrumPad import *
from Live.Eq8Device import *
from Live.Groove import *
from Live.GroovePool import *
from Live.HybridReverbDevice import *
from Live.Listener import *
from Live.LomObject import *
from Live.MaxDevice import *
from Live.MeldDevice import *
from Live.MidiMap import *
from Live.MixerDevice import *
from Live.PluginDevice import *
from Live.RackDevice import *
from Live.RoarDevice import *
from Live.Sample import *
from Live.Scene import *
from Live.ShifterDevice import *
from Live.SimplerDevice import *
from Live.Song import *
from Live.SpectralResonatorDevice import *
from Live.WavetableDevice import *


class DeviceContainer:
    """
    This class is a common super class of Track and Chain
    """


class DeviceInsertMode:
    """
    Class that represent an enumeration of values for DeviceInsertMode
    int([x]) -> integer
    int(x, base=10) -> integer

    Convert a number or string to an integer, or return 0 if no arguments
    are given.  If x is a number, return x.__int__().  For floating point
    numbers, this truncates towards zero.

    If x is not a number or if base is given, then x must be a string,
    bytes, or bytearray instance representing an integer literal in the
    given base.  The literal can be preceded by '+' or '-' and be surrounded
    by whitespace.  The base defaults to 10.  Valid bases are 0 and 2-36.
    Base 0 means to interpret the base from the string as an integer literal.
    >>> int('0b100', base=0)
    4
    """

    default = 0
    selected_left = 1
    selected_right = 2
    count = 3


class RoutingChannel:
    """
    This class represents a routing channel.
    """

    @property
    def display_name(self) -> str:
        """
        Display name of routing channel.
        """

    @property
    def layout(self) -> Any:
        """
        The routing channel's Layout, e.g., mono or stereo.
        """


class RoutingChannelLayout:
    """
    Class that represent an enumeration of values for RoutingChannelLayout
    int([x]) -> integer
    int(x, base=10) -> integer

    Convert a number or string to an integer, or return 0 if no arguments
    are given.  If x is a number, return x.__int__().  For floating point
    numbers, this truncates towards zero.

    If x is not a number or if base is given, then x must be a string,
    bytes, or bytearray instance representing an integer literal in the
    given base.  The literal can be preceded by '+' or '-' and be surrounded
    by whitespace.  The base defaults to 10.  Valid bases are 0 and 2-36.
    Base 0 means to interpret the base from the string as an integer literal.
    >>> int('0b100', base=0)
    4
    """

    mono = 0
    stereo = 1
    midi = 2


class RoutingChannelVector:
    """
    A container for returning routing channels from Live.
    """

    def append(self, arg2: object) -> None:
        """
        append( (RoutingChannelVector)arg1, (object)arg2) -> None :

            C++ signature :
                void append(std::__1::vector<NRoutingApi::TRoutingChannel, std::__1::allocator<NRoutingApi::TRoutingChannel>> {lvalue},boost::python::api::object)
        """

    def extend(self, arg2: object) -> None:
        """
        extend( (RoutingChannelVector)arg1, (object)arg2) -> None :

            C++ signature :
                void extend(std::__1::vector<NRoutingApi::TRoutingChannel, std::__1::allocator<NRoutingApi::TRoutingChannel>> {lvalue},boost::python::api::object)
        """


class RoutingType:
    """
    This class represents a routing type.
    """

    @property
    def attached_object(self) -> Any:
        """
        Live object associated with the routing type.
        """

    @property
    def category(self) -> Any:
        """
        Category of the routing type.
        """

    @property
    def display_name(self) -> str:
        """
        Display name of routing type.
        """


class RoutingTypeCategory:
    """
    Class that represent an enumeration of values for RoutingTypeCategory
    int([x]) -> integer
    int(x, base=10) -> integer

    Convert a number or string to an integer, or return 0 if no arguments
    are given.  If x is a number, return x.__int__().  For floating point
    numbers, this truncates towards zero.

    If x is not a number or if base is given, then x must be a string,
    bytes, or bytearray instance representing an integer literal in the
    given base.  The literal can be preceded by '+' or '-' and be surrounded
    by whitespace.  The base defaults to 10.  Valid bases are 0 and 2-36.
    Base 0 means to interpret the base from the string as an integer literal.
    >>> int('0b100', base=0)
    4
    """

    external = 0
    rewire = 1
    resampling = 2
    master = 3
    track = 4
    parent_group_track = 5
    none = 6
    invalid = 7


class RoutingTypeVector:
    """
    A container for returning routing types from Live.
    """

    def append(self, arg2: object) -> None:
        """
        append( (RoutingTypeVector)arg1, (object)arg2) -> None :

            C++ signature :
                void append(std::__1::vector<NRoutingApi::TRoutingType, std::__1::allocator<NRoutingApi::TRoutingType>> {lvalue},boost::python::api::object)
        """

    def extend(self, arg2: object) -> None:
        """
        extend( (RoutingTypeVector)arg1, (object)arg2) -> None :

            C++ signature :
                void extend(std::__1::vector<NRoutingApi::TRoutingType, std::__1::allocator<NRoutingApi::TRoutingType>> {lvalue},boost::python::api::object)
        """


class Track:
    """
    This class represents a track in Live. It can be either an Audio
    track, a MIDI Track, a Return Track or the Main track. The Main
    Track and at least one Audio or MIDI track will be always present.
    Return Tracks are optional.
    """

    class View:
        """
        Representing the view aspects of a Track.
        """

        def add_device_insert_mode_listener(self, listener: Callable) -> None:
            """
            add_device_insert_mode_listener( (View)arg1, (object)arg2) -> None :
                Add a listener function or method, which will be called as soon as the
                property "device_insert_mode" has changed.

                C++ signature :
                    void add_device_insert_mode_listener(TPyViewData<ATrack>,boost::python::api::object)
            """

        def add_is_collapsed_listener(self, listener: Callable) -> None:
            """
            add_is_collapsed_listener( (View)arg1, (object)arg2) -> None :
                Add a listener function or method, which will be called as soon as the
                property "is_collapsed" has changed.

                C++ signature :
                    void add_is_collapsed_listener(TPyViewData<ATrack>,boost::python::api::object)
            """

        def add_selected_device_listener(self, listener: Callable) -> None:
            """
            add_selected_device_listener( (View)arg1, (object)arg2) -> None :
                Add a listener function or method, which will be called as soon as the
                property "selected_device" has changed.

                C++ signature :
                    void add_selected_device_listener(TPyViewData<ATrack>,boost::python::api::object)
            """

        def device_insert_mode_has_listener(self, listener: Callable) -> bool:
            """
            device_insert_mode_has_listener( (View)arg1, (object)arg2) -> bool :
                Returns true, if the given listener function or method is connected
                to the property "device_insert_mode".

                C++ signature :
                    bool device_insert_mode_has_listener(TPyViewData<ATrack>,boost::python::api::object)
            """

        def is_collapsed_has_listener(self, listener: Callable) -> bool:
            """
            is_collapsed_has_listener( (View)arg1, (object)arg2) -> bool :
                Returns true, if the given listener function or method is connected
                to the property "is_collapsed".

                C++ signature :
                    bool is_collapsed_has_listener(TPyViewData<ATrack>,boost::python::api::object)
            """

        def remove_device_insert_mode_listener(self, listener: Callable) -> None:
            """
            remove_device_insert_mode_listener( (View)arg1, (object)arg2) -> None :
                Remove a previously set listener function or method from
                property "device_insert_mode".

                C++ signature :
                    void remove_device_insert_mode_listener(TPyViewData<ATrack>,boost::python::api::object)
            """

        def remove_is_collapsed_listener(self, listener: Callable) -> None:
            """
            remove_is_collapsed_listener( (View)arg1, (object)arg2) -> None :
                Remove a previously set listener function or method from
                property "is_collapsed".

                C++ signature :
                    void remove_is_collapsed_listener(TPyViewData<ATrack>,boost::python::api::object)
            """

        def remove_selected_device_listener(self, listener: Callable) -> None:
            """
            remove_selected_device_listener( (View)arg1, (object)arg2) -> None :
                Remove a previously set listener function or method from
                property "selected_device".

                C++ signature :
                    void remove_selected_device_listener(TPyViewData<ATrack>,boost::python::api::object)
            """

        def select_instrument(self) -> bool:
            """
            select_instrument( (View)arg1) -> bool :
                Selects the track's instrument if it has one.

                C++ signature :
                    bool select_instrument(TPyViewData<ATrack>)
            """

        def selected_device_has_listener(self, listener: Callable) -> bool:
            """
            selected_device_has_listener( (View)arg1, (object)arg2) -> bool :
                Returns true, if the given listener function or method is connected
                to the property "selected_device".

                C++ signature :
                    bool selected_device_has_listener(TPyViewData<ATrack>,boost::python::api::object)
            """

        @property
        def canonical_parent(self) -> Any:
            """
            Get the canonical parent of the track view.
            """

        @property
        def device_insert_mode(self) -> Any:
            """
            Get/Listen the device insertion mode of the track.  By default, it will insert devices at the end, but it can be changed to make it relative to current selection.
            """

        @property
        def is_collapsed(self) -> bool:
            """
            Get/Set/Listen if the track is shown collapsed in the arranger view.
            """

        @is_collapsed.setter
        def is_collapsed(self, value: bool) -> None:
            pass

        @property
        def selected_device(self) -> Device:
            """
            Get/Set/Listen the insertion mode of the device.  While in insertion mode, loading new devices from the browser will place devices at the selected position.
            """

        @selected_device.setter
        def selected_device(self, value: Device) -> None:
            pass

    class monitoring_states:
        """
        Class that represent an enumeration of values for monitoring_states
        int([x]) -> integer
        int(x, base=10) -> integer

        Convert a number or string to an integer, or return 0 if no arguments
        are given.  If x is a number, return x.__int__().  For floating point
        numbers, this truncates towards zero.

        If x is not a number or if base is given, then x must be a string,
        bytes, or bytearray instance representing an integer literal in the
        given base.  The literal can be preceded by '+' or '-' and be surrounded
        by whitespace.  The base defaults to 10.  Valid bases are 0 and 2-36.
        Base 0 means to interpret the base from the string as an integer literal.
        >>> int('0b100', base=0)
        4
        """

        IN = 0
        AUTO = 1
        OFF = 2

    def add_arm_listener(self, listener: Callable) -> None:
        """
        add_arm_listener( (Track)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "arm" has changed.

            C++ signature :
                void add_arm_listener(TTrackPyHandle,boost::python::api::object)
        """

    def add_arrangement_clips_listener(self, listener: Callable) -> None:
        """
        add_arrangement_clips_listener( (Track)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "arrangement_clips" has changed.

            C++ signature :
                void add_arrangement_clips_listener(TTrackPyHandle,boost::python::api::object)
        """

    def add_available_input_routing_channels_listener(self, listener: Callable) -> None:
        """
        add_available_input_routing_channels_listener( (Track)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "available_input_routing_channels" has changed.

            C++ signature :
                void add_available_input_routing_channels_listener(TTrackPyHandle,boost::python::api::object)
        """

    def add_available_input_routing_types_listener(self, listener: Callable) -> None:
        """
        add_available_input_routing_types_listener( (Track)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "available_input_routing_types" has changed.

            C++ signature :
                void add_available_input_routing_types_listener(TTrackPyHandle,boost::python::api::object)
        """

    def add_available_output_routing_channels_listener(self, listener: Callable) -> None:
        """
        add_available_output_routing_channels_listener( (Track)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "available_output_routing_channels" has changed.

            C++ signature :
                void add_available_output_routing_channels_listener(TTrackPyHandle,boost::python::api::object)
        """

    def add_available_output_routing_types_listener(self, listener: Callable) -> None:
        """
        add_available_output_routing_types_listener( (Track)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "available_output_routing_types" has changed.

            C++ signature :
                void add_available_output_routing_types_listener(TTrackPyHandle,boost::python::api::object)
        """

    def add_clip_slots_listener(self, listener: Callable) -> None:
        """
        add_clip_slots_listener( (Track)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "clip_slots" has changed.

            C++ signature :
                void add_clip_slots_listener(TTrackPyHandle,boost::python::api::object)
        """

    def add_color_index_listener(self, listener: Callable) -> None:
        """
        add_color_index_listener( (Track)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "color_index" has changed.

            C++ signature :
                void add_color_index_listener(TTrackPyHandle,boost::python::api::object)
        """

    def add_color_listener(self, listener: Callable) -> None:
        """
        add_color_listener( (Track)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "color" has changed.

            C++ signature :
                void add_color_listener(TTrackPyHandle,boost::python::api::object)
        """

    def add_current_input_routing_listener(self, listener: Callable) -> None:
        """
        add_current_input_routing_listener( (Track)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "current_input_routing" has changed.

            C++ signature :
                void add_current_input_routing_listener(TTrackPyHandle,boost::python::api::object)
        """

    def add_current_input_sub_routing_listener(self, listener: Callable) -> None:
        """
        add_current_input_sub_routing_listener( (Track)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "current_input_sub_routing" has changed.

            C++ signature :
                void add_current_input_sub_routing_listener(TTrackPyHandle,boost::python::api::object)
        """

    def add_current_monitoring_state_listener(self, listener: Callable) -> None:
        """
        add_current_monitoring_state_listener( (Track)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "current_monitoring_state" has changed.

            C++ signature :
                void add_current_monitoring_state_listener(TTrackPyHandle,boost::python::api::object)
        """

    def add_current_output_routing_listener(self, listener: Callable) -> None:
        """
        add_current_output_routing_listener( (Track)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "current_output_routing" has changed.

            C++ signature :
                void add_current_output_routing_listener(TTrackPyHandle,boost::python::api::object)
        """

    def add_current_output_sub_routing_listener(self, listener: Callable) -> None:
        """
        add_current_output_sub_routing_listener( (Track)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "current_output_sub_routing" has changed.

            C++ signature :
                void add_current_output_sub_routing_listener(TTrackPyHandle,boost::python::api::object)
        """

    def add_data_listener(self, listener: Callable) -> None:
        """
        add_data_listener( (Track)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "data" has changed.

            C++ signature :
                void add_data_listener(TTrackPyHandle,boost::python::api::object)
        """

    def add_devices_listener(self, listener: Callable) -> None:
        """
        add_devices_listener( (Track)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "devices" has changed.

            C++ signature :
                void add_devices_listener(TTrackPyHandle,boost::python::api::object)
        """

    def add_fired_slot_index_listener(self, listener: Callable) -> None:
        """
        add_fired_slot_index_listener( (Track)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "fired_slot_index" has changed.

            C++ signature :
                void add_fired_slot_index_listener(TTrackPyHandle,boost::python::api::object)
        """

    def add_has_audio_input_listener(self, listener: Callable) -> None:
        """
        add_has_audio_input_listener( (Track)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "has_audio_input" has changed.

            C++ signature :
                void add_has_audio_input_listener(TTrackPyHandle,boost::python::api::object)
        """

    def add_has_audio_output_listener(self, listener: Callable) -> None:
        """
        add_has_audio_output_listener( (Track)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "has_audio_output" has changed.

            C++ signature :
                void add_has_audio_output_listener(TTrackPyHandle,boost::python::api::object)
        """

    def add_has_midi_input_listener(self, listener: Callable) -> None:
        """
        add_has_midi_input_listener( (Track)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "has_midi_input" has changed.

            C++ signature :
                void add_has_midi_input_listener(TTrackPyHandle,boost::python::api::object)
        """

    def add_has_midi_output_listener(self, listener: Callable) -> None:
        """
        add_has_midi_output_listener( (Track)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "has_midi_output" has changed.

            C++ signature :
                void add_has_midi_output_listener(TTrackPyHandle,boost::python::api::object)
        """

    def add_implicit_arm_listener(self, listener: Callable) -> None:
        """
        add_implicit_arm_listener( (Track)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "implicit_arm" has changed.

            C++ signature :
                void add_implicit_arm_listener(TTrackPyHandle,boost::python::api::object)
        """

    def add_input_meter_left_listener(self, listener: Callable) -> None:
        """
        add_input_meter_left_listener( (Track)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "input_meter_left" has changed.

            C++ signature :
                void add_input_meter_left_listener(TTrackPyHandle,boost::python::api::object)
        """

    def add_input_meter_level_listener(self, listener: Callable) -> None:
        """
        add_input_meter_level_listener( (Track)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "input_meter_level" has changed.

            C++ signature :
                void add_input_meter_level_listener(TTrackPyHandle,boost::python::api::object)
        """

    def add_input_meter_right_listener(self, listener: Callable) -> None:
        """
        add_input_meter_right_listener( (Track)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "input_meter_right" has changed.

            C++ signature :
                void add_input_meter_right_listener(TTrackPyHandle,boost::python::api::object)
        """

    def add_input_routing_channel_listener(self, listener: Callable) -> None:
        """
        add_input_routing_channel_listener( (Track)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "input_routing_channel" has changed.

            C++ signature :
                void add_input_routing_channel_listener(TTrackPyHandle,boost::python::api::object)
        """

    def add_input_routing_type_listener(self, listener: Callable) -> None:
        """
        add_input_routing_type_listener( (Track)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "input_routing_type" has changed.

            C++ signature :
                void add_input_routing_type_listener(TTrackPyHandle,boost::python::api::object)
        """

    def add_input_routings_listener(self, listener: Callable) -> None:
        """
        add_input_routings_listener( (Track)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "input_routings" has changed.

            C++ signature :
                void add_input_routings_listener(TTrackPyHandle,boost::python::api::object)
        """

    def add_input_sub_routings_listener(self, listener: Callable) -> None:
        """
        add_input_sub_routings_listener( (Track)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "input_sub_routings" has changed.

            C++ signature :
                void add_input_sub_routings_listener(TTrackPyHandle,boost::python::api::object)
        """

    def add_is_frozen_listener(self, listener: Callable) -> None:
        """
        add_is_frozen_listener( (Track)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "is_frozen" has changed.

            C++ signature :
                void add_is_frozen_listener(TTrackPyHandle,boost::python::api::object)
        """

    def add_is_showing_chains_listener(self, listener: Callable) -> None:
        """
        add_is_showing_chains_listener( (Track)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "is_showing_chains" has changed.

            C++ signature :
                void add_is_showing_chains_listener(TTrackPyHandle,boost::python::api::object)
        """

    def add_mute_listener(self, listener: Callable) -> None:
        """
        add_mute_listener( (Track)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "mute" has changed.

            C++ signature :
                void add_mute_listener(TTrackPyHandle,boost::python::api::object)
        """

    def add_muted_via_solo_listener(self, listener: Callable) -> None:
        """
        add_muted_via_solo_listener( (Track)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "muted_via_solo" has changed.

            C++ signature :
                void add_muted_via_solo_listener(TTrackPyHandle,boost::python::api::object)
        """

    def add_name_listener(self, listener: Callable) -> None:
        """
        add_name_listener( (Track)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "name" has changed.

            C++ signature :
                void add_name_listener(TTrackPyHandle,boost::python::api::object)
        """

    def add_output_meter_left_listener(self, listener: Callable) -> None:
        """
        add_output_meter_left_listener( (Track)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "output_meter_left" has changed.

            C++ signature :
                void add_output_meter_left_listener(TTrackPyHandle,boost::python::api::object)
        """

    def add_output_meter_level_listener(self, listener: Callable) -> None:
        """
        add_output_meter_level_listener( (Track)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "output_meter_level" has changed.

            C++ signature :
                void add_output_meter_level_listener(TTrackPyHandle,boost::python::api::object)
        """

    def add_output_meter_right_listener(self, listener: Callable) -> None:
        """
        add_output_meter_right_listener( (Track)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "output_meter_right" has changed.

            C++ signature :
                void add_output_meter_right_listener(TTrackPyHandle,boost::python::api::object)
        """

    def add_output_routing_channel_listener(self, listener: Callable) -> None:
        """
        add_output_routing_channel_listener( (Track)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "output_routing_channel" has changed.

            C++ signature :
                void add_output_routing_channel_listener(TTrackPyHandle,boost::python::api::object)
        """

    def add_output_routing_type_listener(self, listener: Callable) -> None:
        """
        add_output_routing_type_listener( (Track)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "output_routing_type" has changed.

            C++ signature :
                void add_output_routing_type_listener(TTrackPyHandle,boost::python::api::object)
        """

    def add_output_routings_listener(self, listener: Callable) -> None:
        """
        add_output_routings_listener( (Track)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "output_routings" has changed.

            C++ signature :
                void add_output_routings_listener(TTrackPyHandle,boost::python::api::object)
        """

    def add_output_sub_routings_listener(self, listener: Callable) -> None:
        """
        add_output_sub_routings_listener( (Track)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "output_sub_routings" has changed.

            C++ signature :
                void add_output_sub_routings_listener(TTrackPyHandle,boost::python::api::object)
        """

    def add_performance_impact_listener(self, listener: Callable) -> None:
        """
        add_performance_impact_listener( (Track)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "performance_impact" has changed.

            C++ signature :
                void add_performance_impact_listener(TTrackPyHandle,boost::python::api::object)
        """

    def add_playing_slot_index_listener(self, listener: Callable) -> None:
        """
        add_playing_slot_index_listener( (Track)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "playing_slot_index" has changed.

            C++ signature :
                void add_playing_slot_index_listener(TTrackPyHandle,boost::python::api::object)
        """

    def add_solo_listener(self, listener: Callable) -> None:
        """
        add_solo_listener( (Track)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "solo" has changed.

            C++ signature :
                void add_solo_listener(TTrackPyHandle,boost::python::api::object)
        """

    def arm_has_listener(self, listener: Callable) -> bool:
        """
        arm_has_listener( (Track)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "arm".

            C++ signature :
                bool arm_has_listener(TTrackPyHandle,boost::python::api::object)
        """

    def arrangement_clips_has_listener(self, listener: Callable) -> bool:
        """
        arrangement_clips_has_listener( (Track)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "arrangement_clips".

            C++ signature :
                bool arrangement_clips_has_listener(TTrackPyHandle,boost::python::api::object)
        """

    def available_input_routing_channels_has_listener(self, listener: Callable) -> bool:
        """
        available_input_routing_channels_has_listener( (Track)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "available_input_routing_channels".

            C++ signature :
                bool available_input_routing_channels_has_listener(TTrackPyHandle,boost::python::api::object)
        """

    def available_input_routing_types_has_listener(self, listener: Callable) -> bool:
        """
        available_input_routing_types_has_listener( (Track)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "available_input_routing_types".

            C++ signature :
                bool available_input_routing_types_has_listener(TTrackPyHandle,boost::python::api::object)
        """

    def available_output_routing_channels_has_listener(self, listener: Callable) -> bool:
        """
        available_output_routing_channels_has_listener( (Track)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "available_output_routing_channels".

            C++ signature :
                bool available_output_routing_channels_has_listener(TTrackPyHandle,boost::python::api::object)
        """

    def available_output_routing_types_has_listener(self, listener: Callable) -> bool:
        """
        available_output_routing_types_has_listener( (Track)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "available_output_routing_types".

            C++ signature :
                bool available_output_routing_types_has_listener(TTrackPyHandle,boost::python::api::object)
        """

    def clip_slots_has_listener(self, listener: Callable) -> bool:
        """
        clip_slots_has_listener( (Track)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "clip_slots".

            C++ signature :
                bool clip_slots_has_listener(TTrackPyHandle,boost::python::api::object)
        """

    def color_has_listener(self, listener: Callable) -> bool:
        """
        color_has_listener( (Track)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "color".

            C++ signature :
                bool color_has_listener(TTrackPyHandle,boost::python::api::object)
        """

    def color_index_has_listener(self, listener: Callable) -> bool:
        """
        color_index_has_listener( (Track)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "color_index".

            C++ signature :
                bool color_index_has_listener(TTrackPyHandle,boost::python::api::object)
        """

    def current_input_routing_has_listener(self, listener: Callable) -> bool:
        """
        current_input_routing_has_listener( (Track)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "current_input_routing".

            C++ signature :
                bool current_input_routing_has_listener(TTrackPyHandle,boost::python::api::object)
        """

    def current_input_sub_routing_has_listener(self, listener: Callable) -> bool:
        """
        current_input_sub_routing_has_listener( (Track)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "current_input_sub_routing".

            C++ signature :
                bool current_input_sub_routing_has_listener(TTrackPyHandle,boost::python::api::object)
        """

    def current_monitoring_state_has_listener(self, listener: Callable) -> bool:
        """
        current_monitoring_state_has_listener( (Track)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "current_monitoring_state".

            C++ signature :
                bool current_monitoring_state_has_listener(TTrackPyHandle,boost::python::api::object)
        """

    def current_output_routing_has_listener(self, listener: Callable) -> bool:
        """
        current_output_routing_has_listener( (Track)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "current_output_routing".

            C++ signature :
                bool current_output_routing_has_listener(TTrackPyHandle,boost::python::api::object)
        """

    def current_output_sub_routing_has_listener(self, listener: Callable) -> bool:
        """
        current_output_sub_routing_has_listener( (Track)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "current_output_sub_routing".

            C++ signature :
                bool current_output_sub_routing_has_listener(TTrackPyHandle,boost::python::api::object)
        """

    def data_has_listener(self, listener: Callable) -> bool:
        """
        data_has_listener( (Track)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "data".

            C++ signature :
                bool data_has_listener(TTrackPyHandle,boost::python::api::object)
        """

    def delete_clip(self, arg2: object) -> None:
        """
        delete_clip( (Track)arg1, (Clip)arg2) -> None :
            Delete the given clip. Raises a runtime error when the clip belongs to another track.

            C++ signature :
                void delete_clip(TTrackPyHandle,TPyHandle<AClip>)
        """

    def delete_device(self, arg2: int) -> None:
        """
        delete_device( (Track)arg1, (int)arg2) -> None :
            Delete a device identified by the index in the 'devices' list.

            C++ signature :
                void delete_device(TTrackPyHandle,int)
        """

    def devices_has_listener(self, listener: Callable) -> bool:
        """
        devices_has_listener( (Track)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "devices".

            C++ signature :
                bool devices_has_listener(TTrackPyHandle,boost::python::api::object)
        """

    def duplicate_clip_slot(self, arg2: int) -> int:
        """
        duplicate_clip_slot( (Track)arg1, (int)arg2) -> int :
            Duplicate a clip and put it into the next free slot and return the index
            of the destination slot. A new scene is created if no free slot is
            available. If creating the new scene would exceed the limitations,
            a runtime error is raised.

            C++ signature :
                int duplicate_clip_slot(TTrackPyHandle,int)
        """

    def duplicate_clip_to_arrangement(self, clip: object, destination_time: float) -> Clip:
        """
        duplicate_clip_to_arrangement( (Track)self, (Clip)clip, (float)destination_time) -> Clip :
            Duplicate the given clip into the arrangement of this track at the provided
            destination time and return it. When the type of the clip and the type of the
            track are incompatible, a runtime error is raised.

            C++ signature :
                TWeakPtr<TPyHandle<AClip>> duplicate_clip_to_arrangement(TTrackPyHandle,TPyHandle<AClip>,double)
        """

    def fired_slot_index_has_listener(self, listener: Callable) -> bool:
        """
        fired_slot_index_has_listener( (Track)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "fired_slot_index".

            C++ signature :
                bool fired_slot_index_has_listener(TTrackPyHandle,boost::python::api::object)
        """

    def get_data(self, key: object, default_value: object) -> object:
        """
        get_data( (Track)arg1, (object)key, (object)default_value) -> object :
            Get data for the given key, that was previously stored using set_data.

            C++ signature :
                boost::python::api::object get_data(TTrackPyHandle,TString,boost::python::api::object)
        """

    def has_audio_input_has_listener(self, listener: Callable) -> bool:
        """
        has_audio_input_has_listener( (Track)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "has_audio_input".

            C++ signature :
                bool has_audio_input_has_listener(TTrackPyHandle,boost::python::api::object)
        """

    def has_audio_output_has_listener(self, listener: Callable) -> bool:
        """
        has_audio_output_has_listener( (Track)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "has_audio_output".

            C++ signature :
                bool has_audio_output_has_listener(TTrackPyHandle,boost::python::api::object)
        """

    def has_midi_input_has_listener(self, listener: Callable) -> bool:
        """
        has_midi_input_has_listener( (Track)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "has_midi_input".

            C++ signature :
                bool has_midi_input_has_listener(TTrackPyHandle,boost::python::api::object)
        """

    def has_midi_output_has_listener(self, listener: Callable) -> bool:
        """
        has_midi_output_has_listener( (Track)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "has_midi_output".

            C++ signature :
                bool has_midi_output_has_listener(TTrackPyHandle,boost::python::api::object)
        """

    def implicit_arm_has_listener(self, listener: Callable) -> bool:
        """
        implicit_arm_has_listener( (Track)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "implicit_arm".

            C++ signature :
                bool implicit_arm_has_listener(TTrackPyHandle,boost::python::api::object)
        """

    def input_meter_left_has_listener(self, listener: Callable) -> bool:
        """
        input_meter_left_has_listener( (Track)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "input_meter_left".

            C++ signature :
                bool input_meter_left_has_listener(TTrackPyHandle,boost::python::api::object)
        """

    def input_meter_level_has_listener(self, listener: Callable) -> bool:
        """
        input_meter_level_has_listener( (Track)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "input_meter_level".

            C++ signature :
                bool input_meter_level_has_listener(TTrackPyHandle,boost::python::api::object)
        """

    def input_meter_right_has_listener(self, listener: Callable) -> bool:
        """
        input_meter_right_has_listener( (Track)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "input_meter_right".

            C++ signature :
                bool input_meter_right_has_listener(TTrackPyHandle,boost::python::api::object)
        """

    def input_routing_channel_has_listener(self, listener: Callable) -> bool:
        """
        input_routing_channel_has_listener( (Track)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "input_routing_channel".

            C++ signature :
                bool input_routing_channel_has_listener(TTrackPyHandle,boost::python::api::object)
        """

    def input_routing_type_has_listener(self, listener: Callable) -> bool:
        """
        input_routing_type_has_listener( (Track)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "input_routing_type".

            C++ signature :
                bool input_routing_type_has_listener(TTrackPyHandle,boost::python::api::object)
        """

    def input_routings_has_listener(self, listener: Callable) -> bool:
        """
        input_routings_has_listener( (Track)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "input_routings".

            C++ signature :
                bool input_routings_has_listener(TTrackPyHandle,boost::python::api::object)
        """

    def input_sub_routings_has_listener(self, listener: Callable) -> bool:
        """
        input_sub_routings_has_listener( (Track)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "input_sub_routings".

            C++ signature :
                bool input_sub_routings_has_listener(TTrackPyHandle,boost::python::api::object)
        """

    def is_frozen_has_listener(self, listener: Callable) -> bool:
        """
        is_frozen_has_listener( (Track)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "is_frozen".

            C++ signature :
                bool is_frozen_has_listener(TTrackPyHandle,boost::python::api::object)
        """

    def is_showing_chains_has_listener(self, listener: Callable) -> bool:
        """
        is_showing_chains_has_listener( (Track)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "is_showing_chains".

            C++ signature :
                bool is_showing_chains_has_listener(TTrackPyHandle,boost::python::api::object)
        """

    def jump_in_running_session_clip(self, arg2: float) -> None:
        """
        jump_in_running_session_clip( (Track)arg1, (float)arg2) -> None :
            Jump forward or backward in the currently running Sessionclip (if any)
            by the specified relative amount in beats. Does nothing if no Session Clip
            is currently running.

            C++ signature :
                void jump_in_running_session_clip(TTrackPyHandle,double)
        """

    def mute_has_listener(self, listener: Callable) -> bool:
        """
        mute_has_listener( (Track)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "mute".

            C++ signature :
                bool mute_has_listener(TTrackPyHandle,boost::python::api::object)
        """

    def muted_via_solo_has_listener(self, listener: Callable) -> bool:
        """
        muted_via_solo_has_listener( (Track)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "muted_via_solo".

            C++ signature :
                bool muted_via_solo_has_listener(TTrackPyHandle,boost::python::api::object)
        """

    def name_has_listener(self, listener: Callable) -> bool:
        """
        name_has_listener( (Track)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "name".

            C++ signature :
                bool name_has_listener(TTrackPyHandle,boost::python::api::object)
        """

    def output_meter_left_has_listener(self, listener: Callable) -> bool:
        """
        output_meter_left_has_listener( (Track)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "output_meter_left".

            C++ signature :
                bool output_meter_left_has_listener(TTrackPyHandle,boost::python::api::object)
        """

    def output_meter_level_has_listener(self, listener: Callable) -> bool:
        """
        output_meter_level_has_listener( (Track)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "output_meter_level".

            C++ signature :
                bool output_meter_level_has_listener(TTrackPyHandle,boost::python::api::object)
        """

    def output_meter_right_has_listener(self, listener: Callable) -> bool:
        """
        output_meter_right_has_listener( (Track)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "output_meter_right".

            C++ signature :
                bool output_meter_right_has_listener(TTrackPyHandle,boost::python::api::object)
        """

    def output_routing_channel_has_listener(self, listener: Callable) -> bool:
        """
        output_routing_channel_has_listener( (Track)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "output_routing_channel".

            C++ signature :
                bool output_routing_channel_has_listener(TTrackPyHandle,boost::python::api::object)
        """

    def output_routing_type_has_listener(self, listener: Callable) -> bool:
        """
        output_routing_type_has_listener( (Track)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "output_routing_type".

            C++ signature :
                bool output_routing_type_has_listener(TTrackPyHandle,boost::python::api::object)
        """

    def output_routings_has_listener(self, listener: Callable) -> bool:
        """
        output_routings_has_listener( (Track)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "output_routings".

            C++ signature :
                bool output_routings_has_listener(TTrackPyHandle,boost::python::api::object)
        """

    def output_sub_routings_has_listener(self, listener: Callable) -> bool:
        """
        output_sub_routings_has_listener( (Track)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "output_sub_routings".

            C++ signature :
                bool output_sub_routings_has_listener(TTrackPyHandle,boost::python::api::object)
        """

    def performance_impact_has_listener(self, listener: Callable) -> bool:
        """
        performance_impact_has_listener( (Track)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "performance_impact".

            C++ signature :
                bool performance_impact_has_listener(TTrackPyHandle,boost::python::api::object)
        """

    def playing_slot_index_has_listener(self, listener: Callable) -> bool:
        """
        playing_slot_index_has_listener( (Track)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "playing_slot_index".

            C++ signature :
                bool playing_slot_index_has_listener(TTrackPyHandle,boost::python::api::object)
        """

    def remove_arm_listener(self, listener: Callable) -> None:
        """
        remove_arm_listener( (Track)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "arm".

            C++ signature :
                void remove_arm_listener(TTrackPyHandle,boost::python::api::object)
        """

    def remove_arrangement_clips_listener(self, listener: Callable) -> None:
        """
        remove_arrangement_clips_listener( (Track)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "arrangement_clips".

            C++ signature :
                void remove_arrangement_clips_listener(TTrackPyHandle,boost::python::api::object)
        """

    def remove_available_input_routing_channels_listener(self, listener: Callable) -> None:
        """
        remove_available_input_routing_channels_listener( (Track)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "available_input_routing_channels".

            C++ signature :
                void remove_available_input_routing_channels_listener(TTrackPyHandle,boost::python::api::object)
        """

    def remove_available_input_routing_types_listener(self, listener: Callable) -> None:
        """
        remove_available_input_routing_types_listener( (Track)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "available_input_routing_types".

            C++ signature :
                void remove_available_input_routing_types_listener(TTrackPyHandle,boost::python::api::object)
        """

    def remove_available_output_routing_channels_listener(self, listener: Callable) -> None:
        """
        remove_available_output_routing_channels_listener( (Track)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "available_output_routing_channels".

            C++ signature :
                void remove_available_output_routing_channels_listener(TTrackPyHandle,boost::python::api::object)
        """

    def remove_available_output_routing_types_listener(self, listener: Callable) -> None:
        """
        remove_available_output_routing_types_listener( (Track)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "available_output_routing_types".

            C++ signature :
                void remove_available_output_routing_types_listener(TTrackPyHandle,boost::python::api::object)
        """

    def remove_clip_slots_listener(self, listener: Callable) -> None:
        """
        remove_clip_slots_listener( (Track)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "clip_slots".

            C++ signature :
                void remove_clip_slots_listener(TTrackPyHandle,boost::python::api::object)
        """

    def remove_color_index_listener(self, listener: Callable) -> None:
        """
        remove_color_index_listener( (Track)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "color_index".

            C++ signature :
                void remove_color_index_listener(TTrackPyHandle,boost::python::api::object)
        """

    def remove_color_listener(self, listener: Callable) -> None:
        """
        remove_color_listener( (Track)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "color".

            C++ signature :
                void remove_color_listener(TTrackPyHandle,boost::python::api::object)
        """

    def remove_current_input_routing_listener(self, listener: Callable) -> None:
        """
        remove_current_input_routing_listener( (Track)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "current_input_routing".

            C++ signature :
                void remove_current_input_routing_listener(TTrackPyHandle,boost::python::api::object)
        """

    def remove_current_input_sub_routing_listener(self, listener: Callable) -> None:
        """
        remove_current_input_sub_routing_listener( (Track)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "current_input_sub_routing".

            C++ signature :
                void remove_current_input_sub_routing_listener(TTrackPyHandle,boost::python::api::object)
        """

    def remove_current_monitoring_state_listener(self, listener: Callable) -> None:
        """
        remove_current_monitoring_state_listener( (Track)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "current_monitoring_state".

            C++ signature :
                void remove_current_monitoring_state_listener(TTrackPyHandle,boost::python::api::object)
        """

    def remove_current_output_routing_listener(self, listener: Callable) -> None:
        """
        remove_current_output_routing_listener( (Track)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "current_output_routing".

            C++ signature :
                void remove_current_output_routing_listener(TTrackPyHandle,boost::python::api::object)
        """

    def remove_current_output_sub_routing_listener(self, listener: Callable) -> None:
        """
        remove_current_output_sub_routing_listener( (Track)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "current_output_sub_routing".

            C++ signature :
                void remove_current_output_sub_routing_listener(TTrackPyHandle,boost::python::api::object)
        """

    def remove_data_listener(self, listener: Callable) -> None:
        """
        remove_data_listener( (Track)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "data".

            C++ signature :
                void remove_data_listener(TTrackPyHandle,boost::python::api::object)
        """

    def remove_devices_listener(self, listener: Callable) -> None:
        """
        remove_devices_listener( (Track)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "devices".

            C++ signature :
                void remove_devices_listener(TTrackPyHandle,boost::python::api::object)
        """

    def remove_fired_slot_index_listener(self, listener: Callable) -> None:
        """
        remove_fired_slot_index_listener( (Track)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "fired_slot_index".

            C++ signature :
                void remove_fired_slot_index_listener(TTrackPyHandle,boost::python::api::object)
        """

    def remove_has_audio_input_listener(self, listener: Callable) -> None:
        """
        remove_has_audio_input_listener( (Track)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "has_audio_input".

            C++ signature :
                void remove_has_audio_input_listener(TTrackPyHandle,boost::python::api::object)
        """

    def remove_has_audio_output_listener(self, listener: Callable) -> None:
        """
        remove_has_audio_output_listener( (Track)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "has_audio_output".

            C++ signature :
                void remove_has_audio_output_listener(TTrackPyHandle,boost::python::api::object)
        """

    def remove_has_midi_input_listener(self, listener: Callable) -> None:
        """
        remove_has_midi_input_listener( (Track)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "has_midi_input".

            C++ signature :
                void remove_has_midi_input_listener(TTrackPyHandle,boost::python::api::object)
        """

    def remove_has_midi_output_listener(self, listener: Callable) -> None:
        """
        remove_has_midi_output_listener( (Track)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "has_midi_output".

            C++ signature :
                void remove_has_midi_output_listener(TTrackPyHandle,boost::python::api::object)
        """

    def remove_implicit_arm_listener(self, listener: Callable) -> None:
        """
        remove_implicit_arm_listener( (Track)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "implicit_arm".

            C++ signature :
                void remove_implicit_arm_listener(TTrackPyHandle,boost::python::api::object)
        """

    def remove_input_meter_left_listener(self, listener: Callable) -> None:
        """
        remove_input_meter_left_listener( (Track)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "input_meter_left".

            C++ signature :
                void remove_input_meter_left_listener(TTrackPyHandle,boost::python::api::object)
        """

    def remove_input_meter_level_listener(self, listener: Callable) -> None:
        """
        remove_input_meter_level_listener( (Track)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "input_meter_level".

            C++ signature :
                void remove_input_meter_level_listener(TTrackPyHandle,boost::python::api::object)
        """

    def remove_input_meter_right_listener(self, listener: Callable) -> None:
        """
        remove_input_meter_right_listener( (Track)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "input_meter_right".

            C++ signature :
                void remove_input_meter_right_listener(TTrackPyHandle,boost::python::api::object)
        """

    def remove_input_routing_channel_listener(self, listener: Callable) -> None:
        """
        remove_input_routing_channel_listener( (Track)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "input_routing_channel".

            C++ signature :
                void remove_input_routing_channel_listener(TTrackPyHandle,boost::python::api::object)
        """

    def remove_input_routing_type_listener(self, listener: Callable) -> None:
        """
        remove_input_routing_type_listener( (Track)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "input_routing_type".

            C++ signature :
                void remove_input_routing_type_listener(TTrackPyHandle,boost::python::api::object)
        """

    def remove_input_routings_listener(self, listener: Callable) -> None:
        """
        remove_input_routings_listener( (Track)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "input_routings".

            C++ signature :
                void remove_input_routings_listener(TTrackPyHandle,boost::python::api::object)
        """

    def remove_input_sub_routings_listener(self, listener: Callable) -> None:
        """
        remove_input_sub_routings_listener( (Track)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "input_sub_routings".

            C++ signature :
                void remove_input_sub_routings_listener(TTrackPyHandle,boost::python::api::object)
        """

    def remove_is_frozen_listener(self, listener: Callable) -> None:
        """
        remove_is_frozen_listener( (Track)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "is_frozen".

            C++ signature :
                void remove_is_frozen_listener(TTrackPyHandle,boost::python::api::object)
        """

    def remove_is_showing_chains_listener(self, listener: Callable) -> None:
        """
        remove_is_showing_chains_listener( (Track)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "is_showing_chains".

            C++ signature :
                void remove_is_showing_chains_listener(TTrackPyHandle,boost::python::api::object)
        """

    def remove_mute_listener(self, listener: Callable) -> None:
        """
        remove_mute_listener( (Track)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "mute".

            C++ signature :
                void remove_mute_listener(TTrackPyHandle,boost::python::api::object)
        """

    def remove_muted_via_solo_listener(self, listener: Callable) -> None:
        """
        remove_muted_via_solo_listener( (Track)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "muted_via_solo".

            C++ signature :
                void remove_muted_via_solo_listener(TTrackPyHandle,boost::python::api::object)
        """

    def remove_name_listener(self, listener: Callable) -> None:
        """
        remove_name_listener( (Track)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "name".

            C++ signature :
                void remove_name_listener(TTrackPyHandle,boost::python::api::object)
        """

    def remove_output_meter_left_listener(self, listener: Callable) -> None:
        """
        remove_output_meter_left_listener( (Track)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "output_meter_left".

            C++ signature :
                void remove_output_meter_left_listener(TTrackPyHandle,boost::python::api::object)
        """

    def remove_output_meter_level_listener(self, listener: Callable) -> None:
        """
        remove_output_meter_level_listener( (Track)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "output_meter_level".

            C++ signature :
                void remove_output_meter_level_listener(TTrackPyHandle,boost::python::api::object)
        """

    def remove_output_meter_right_listener(self, listener: Callable) -> None:
        """
        remove_output_meter_right_listener( (Track)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "output_meter_right".

            C++ signature :
                void remove_output_meter_right_listener(TTrackPyHandle,boost::python::api::object)
        """

    def remove_output_routing_channel_listener(self, listener: Callable) -> None:
        """
        remove_output_routing_channel_listener( (Track)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "output_routing_channel".

            C++ signature :
                void remove_output_routing_channel_listener(TTrackPyHandle,boost::python::api::object)
        """

    def remove_output_routing_type_listener(self, listener: Callable) -> None:
        """
        remove_output_routing_type_listener( (Track)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "output_routing_type".

            C++ signature :
                void remove_output_routing_type_listener(TTrackPyHandle,boost::python::api::object)
        """

    def remove_output_routings_listener(self, listener: Callable) -> None:
        """
        remove_output_routings_listener( (Track)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "output_routings".

            C++ signature :
                void remove_output_routings_listener(TTrackPyHandle,boost::python::api::object)
        """

    def remove_output_sub_routings_listener(self, listener: Callable) -> None:
        """
        remove_output_sub_routings_listener( (Track)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "output_sub_routings".

            C++ signature :
                void remove_output_sub_routings_listener(TTrackPyHandle,boost::python::api::object)
        """

    def remove_performance_impact_listener(self, listener: Callable) -> None:
        """
        remove_performance_impact_listener( (Track)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "performance_impact".

            C++ signature :
                void remove_performance_impact_listener(TTrackPyHandle,boost::python::api::object)
        """

    def remove_playing_slot_index_listener(self, listener: Callable) -> None:
        """
        remove_playing_slot_index_listener( (Track)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "playing_slot_index".

            C++ signature :
                void remove_playing_slot_index_listener(TTrackPyHandle,boost::python::api::object)
        """

    def remove_solo_listener(self, listener: Callable) -> None:
        """
        remove_solo_listener( (Track)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "solo".

            C++ signature :
                void remove_solo_listener(TTrackPyHandle,boost::python::api::object)
        """

    def set_data(self, key: object, value: object) -> None:
        """
        set_data( (Track)arg1, (object)key, (object)value) -> None :
            Store data for the given key in this object. The data is persistent and will be restored when loading the Live Set.

            C++ signature :
                void set_data(TTrackPyHandle,TString,boost::python::api::object)
        """

    def solo_has_listener(self, listener: Callable) -> bool:
        """
        solo_has_listener( (Track)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "solo".

            C++ signature :
                bool solo_has_listener(TTrackPyHandle,boost::python::api::object)
        """

    def stop_all_clips(self, Quantized: bool = True) -> None:
        """
        stop_all_clips( (Track)arg1 [, (bool)Quantized=True]) -> None :
            Stop running and triggered clip and slots on this track.

            C++ signature :
                void stop_all_clips(TTrackPyHandle [,bool=True])
        """

    @property
    def arm(self) -> Any:
        """
        Arm the track for recording. Not available for Main- and Send Tracks.
        """

    @property
    def arrangement_clips(self) -> list[Clip]:
        """
        const access to the list of clips in arrangement viewThe list will be empty for the main, send and group tracks.
        """

    @property
    def available_input_routing_channels(self) -> Any:
        """
        Return a list of source channels for input routing.
        """

    @property
    def available_input_routing_types(self) -> Any:
        """
        Return a list of source types for input routing.
        """

    @property
    def available_output_routing_channels(self) -> Any:
        """
        Return a list of destination channels for output routing.
        """

    @property
    def available_output_routing_types(self) -> Any:
        """
        Return a list of destination types for output routing.
        """

    @property
    def can_be_armed(self) -> bool:
        """
        return True, if this Track has a valid arm property. Not all tracks
        can be armed (for example return Tracks or the Main Tracks).
        """

    @property
    def can_be_frozen(self) -> bool:
        """
        return True, if this Track can be frozen.
        """

    @property
    def can_show_chains(self) -> bool:
        """
        return True, if this Track contains a rack instrument device that is capable of showing its chains in session view.
        """

    @property
    def canonical_parent(self) -> Any:
        """
        Get the canonical parent of the track.
        """

    @property
    def clip_slots(self) -> list[ClipSlot]:
        """
        const access to the list of clipslots (see class AClipSlot) for this track.
        The list will be empty for the main and sendtracks.
        """

    @property
    def color(self) -> Any:
        """
        Get/set access to the color of the Track (RGB).
        """

    @color.setter
    def color(self, value: Any) -> None:
        pass

    @property
    def color_index(self) -> Any:
        """
        Get/Set access to the color index of the track. Can be None for no color.
        """

    @color_index.setter
    def color_index(self, value: Any) -> None:
        pass

    @property
    def current_input_routing(self) -> Any:
        """
        Get/Set the name of the current active input routing.
        When setting a new routing, the new routing must be one of the available ones.
        """

    @current_input_routing.setter
    def current_input_routing(self, value: Any) -> None:
        pass

    @property
    def current_input_sub_routing(self) -> Any:
        """
        Get/Set the current active input sub routing.
        When setting a new routing, the new routing must be one of the available ones.
        """

    @current_input_sub_routing.setter
    def current_input_sub_routing(self, value: Any) -> None:
        pass

    @property
    def current_monitoring_state(self) -> Any:
        """
        Get/Set the track's current monitoring state.
        """

    @current_monitoring_state.setter
    def current_monitoring_state(self, value: Any) -> None:
        pass

    @property
    def current_output_routing(self) -> Any:
        """
        Get/Set the current active output routing.
        When setting a new routing, the new routing must be one of the available ones.
        """

    @current_output_routing.setter
    def current_output_routing(self, value: Any) -> None:
        pass

    @property
    def current_output_sub_routing(self) -> Any:
        """
        Get/Set the current active output sub routing.
        When setting a new routing, the new routing must be one of the available ones.
        """

    @current_output_sub_routing.setter
    def current_output_sub_routing(self, value: Any) -> None:
        pass

    @property
    def devices(self) -> list[Device]:
        """
        Return const access to all available Devices that are present in the Tracks
        Devicechain. This tuple will also include the 'mixer_device' that every Track
        always has.
        """

    @property
    def fired_slot_index(self) -> Any:
        """
        const access to the index of the fired (and thus blinking) clipslot in this track.
        This index is -1 if no slot is fired and -2 if the track's stop button has been fired.
        """

    @property
    def fold_state(self) -> Any:
        """
        Get/Set whether the track is folded or not. Only available if is_foldable is True.
        """

    @fold_state.setter
    def fold_state(self, value: Any) -> None:
        pass

    @property
    def group_track(self) -> Track:
        """
        return the group track if is_grouped.
        """

    @property
    def has_audio_input(self) -> bool:
        """
        return True, if this Track can be feed with an Audio signal. This is
        true for all Audio Tracks.
        """

    @property
    def has_audio_output(self) -> bool:
        """
        return True, if this Track sends out an Audio signal. This is
        true for all Audio Tracks, and MIDI tracks with an Instrument.
        """

    @property
    def has_midi_input(self) -> bool:
        """
        return True, if this Track can be feed with an Audio signal. This is
        true for all MIDI Tracks.
        """

    @property
    def has_midi_output(self) -> bool:
        """
        return True, if this Track sends out MIDI events. This is
        true for all MIDI Tracks with no Instruments.
        """

    @property
    def implicit_arm(self) -> Any:
        """
        Arm the track for recording. When The track is implicitly armed, it showsin a weaker color in the live GUI and is not saved in the set.
        """

    @property
    def input_meter_left(self) -> Any:
        """
        Momentary value of left input channel meter, 0.0 to 1.0. For Audio Tracks only.
        """

    @property
    def input_meter_level(self) -> Any:
        """
        Return the MIDI or Audio meter value of the Tracks input, depending on the
        type of the Track input. Meter values (MIDI or Audio) are always scaled
        from 0.0 to 1.0.
        """

    @property
    def input_meter_right(self) -> Any:
        """
        Momentary value of right input channel meter, 0.0 to 1.0. For Audio Tracks only.
        """

    @property
    def input_routing_channel(self) -> Any:
        """
        Get and set the current source channel for input routing.
        Raises ValueError if the type isn't one of the current values in
        available_input_routing_channels.
        """

    @property
    def input_routing_type(self) -> Any:
        """
        Get and set the current source type for input routing.
        Raises ValueError if the type isn't one of the current values in
        available_input_routing_types.
        """

    @property
    def input_routings(self) -> Any:
        """
        Const access to the list of available input routings.
        """

    @property
    def input_sub_routings(self) -> Any:
        """
        Return a list of all available input sub routings.
        """

    @property
    def is_foldable(self) -> bool:
        """
        return True if the track can be (un)folded to hide/reveal contained tracks.
        """

    @property
    def is_frozen(self) -> bool:
        """
        return True if this Track is currently frozen. No changes should be applied to the track's devices or clips while it is frozen.
        """

    @property
    def is_grouped(self) -> bool:
        """
        return True if this Track is current part of a group track.
        """

    @property
    def is_part_of_selection(self) -> bool:
        """
        return False if the track is not selected.
        """

    @property
    def is_showing_chains(self) -> bool:
        """
        Get/Set whether a track with a rack device is showing its chains in session view.
        """

    @is_showing_chains.setter
    def is_showing_chains(self, value: bool) -> None:
        pass

    @property
    def is_visible(self) -> bool:
        """
        return False if the track is hidden within a folded group track.
        """

    @property
    def mixer_device(self) -> Device:
        """
        Return access to the special Device that every Track has: This Device contains
        the Volume, Pan, Sendamounts, and Crossfade assignment parameters.
        """

    @property
    def mute(self) -> Any:
        """
        Mute/unmute the track.
        """

    @property
    def muted_via_solo(self) -> Any:
        """
        Returns true if the track is muted because another track is soloed.
        """

    @property
    def name(self) -> str:
        """
        Read/write access to the name of the Track, as visible in the track header.
        """

    @name.setter
    def name(self, value: str) -> None:
        pass

    @property
    def output_meter_left(self) -> Any:
        """
        Momentary value of left output channel meter, 0.0 to 1.0.
        For tracks with audio output only.
        """

    @property
    def output_meter_level(self) -> Any:
        """
        Return the MIDI or Audio meter value of the Track output (behind the
        mixer_device), depending on the type of the Track input, this can be a MIDI
        or Audio meter. Meter values (MIDI or Audio) are always scaled from 0.0 to 1.0.
        """

    @property
    def output_meter_right(self) -> Any:
        """
        Momentary value of right output channel meter, 0.0 to 1.0.
        For tracks with audio output only.
        """

    @property
    def output_routing_channel(self) -> Any:
        """
        Get and set the current destination channel for output routing.
        Raises ValueError if the channel isn't one of the current values in
        available_output_routing_channels.
        """

    @property
    def output_routing_type(self) -> Any:
        """
        Get and set the current destination type for output routing.
        Raises ValueError if the type isn't one of the current values in
        available_output_routing_types.
        """

    @property
    def output_routings(self) -> Any:
        """
        Const access to the list of all available output routings.
        """

    @property
    def output_sub_routings(self) -> Any:
        """
        Return a list of all available output sub routings.
        """

    @property
    def performance_impact(self) -> Any:
        """
        Reports the performance impact of this track.
        """

    @property
    def playing_slot_index(self) -> Any:
        """
        const access to the index of the currently playing clip in the track.
        Will be -1 when no clip is playing.
        """

    @property
    def solo(self) -> Any:
        """
        Get/Set the solo status of the track. Note that this will not disable the
        solo state of any other track. If you want exclusive solo, you have to
        disable the solo state of the other Tracks manually.
        """

    @solo.setter
    def solo(self, value: Any) -> None:
        pass

    @property
    def view(self) -> View:
        """
        Representing the view aspects of a Track.
        """



# decompyle3 version 3.9.0
# Python bytecode version base 3.7.0 (3394)
# Decompiled from: Python 3.7.16 (default, Jan 17 2023, 09:28:58)
# [Clang 14.0.6 ]
# Embedded file name: output/Live/mac_universal_64_static/Release/python-bundle/MIDI Remote Scripts/_MxDCore/LomUtils.py
# Compiled at: 2023-12-21 15:35:35
# Size of source mod 2**32: 14785 bytes
import contextlib
import sys
import types

from ableton.v2.base import liveobj_valid, old_hasattr
from past.builtins import basestring

from .ControlSurfaceWrapper import is_real_control_surface
from .LomTypes import (
    ENUM_TYPES,
    EXTRA_CS_FUNCTIONS,
    PROPERTY_TYPES,
    ROOT_KEYS,
    THIS_DEVICE,
    TUPLE_TYPES,
    LomAttributeError,
    LomObjectError,
    MFLPropertyFormats,
    cs_base_classes,
    get_control_surfaces,
    get_exposed_property_info,
    get_exposed_property_names_for_type,
    get_root_prop,
    is_class,
    is_cplusplus_lom_object,
    is_lom_object,
    is_object_iterable,
)
from .MxDUtils import TupleWrapper


def create_lom_doc_string(lom_object):
    description = ""
    if old_hasattr(lom_object, "__doc__"):
        if isinstance(lom_object.__doc__, basestring):
            if len(lom_object.__doc__) > 0:
                description = "description {}".format(lom_object.__doc__.replace("\n", " ").replace(",", "\\,"))
    return description


class LomInformation:
    def __init__(self, lom_object, epii_version, *a, **k):
        (super().__init__)(*a, **k)
        self._lists_of_children = []
        self._children = []
        self._functions = []
        self._properties = []
        self._description = create_lom_doc_string(lom_object)
        self._generate_object_info(lom_object, epii_version)

    @property
    def description(self):
        return self._description

    @property
    def lists_of_children(self):
        return tuple(self._lists_of_children)

    @property
    def children(self):
        return tuple(self._children)

    @property
    def functions(self):
        return tuple(self._functions)

    @property
    def properties(self):
        return tuple(self._properties)

    def _add_list_of_children(self, prop_name):
        type_name = TUPLE_TYPES[prop_name].__name__
        self._lists_of_children.append((prop_name, type_name))

    def _add_child(self, real_prop, prop_name):
        type_name = (real_prop.__class__ if real_prop is not None else PROPERTY_TYPES[prop_name]).__name__
        self._children.append((prop_name, type_name))

    def _generate_object_info(self, lom_object, epii_version):
        if isinstance(lom_object, cs_base_classes()):
            property_names = list(filter(lambda prop: not prop.startswith("_"), dir(lom_object)))
            functions_implemented_by_mxdcore = EXTRA_CS_FUNCTIONS
        else:
            property_names = get_exposed_property_names_for_type(type(lom_object), epii_version)
            functions_implemented_by_mxdcore = []
        for name in property_names:
            self._generate_property_info(name, lom_object, epii_version)

        if functions_implemented_by_mxdcore:
            for function_name in functions_implemented_by_mxdcore:
                self._functions.append((function_name,))

            self._functions.sort()

    def _generate_property_info(self, prop_name, lom_object, epii_version):
        try:
            real_prop = getattr(lom_object, prop_name)
            if not is_class(real_prop):
                prop_info = get_exposed_property_info(type(lom_object), prop_name, epii_version)
                prop_type = real_prop.__class__.__name__
                if prop_info and prop_info.format == MFLPropertyFormats.JSON:
                    self._properties.append((prop_name, "dict"))
                else:
                    if prop_name in TUPLE_TYPES:
                        self._add_list_of_children(prop_name)
                    else:
                        if prop_name in list(PROPERTY_TYPES.keys()):
                            self._add_child(real_prop, prop_name)
                        else:
                            if prop_name == "canonical_parent":
                                if real_prop is not None:
                                    self._children.append((prop_name, prop_type))
                            else:
                                if callable(real_prop):
                                    if not prop_name.endswith("_listener"):
                                        self._functions.append((prop_name,))
                                else:
                                    if prop_type in ("unicode", "str"):
                                        self._properties.append((prop_name, "str"))
                                    else:
                                        if prop_type not in ("type", "Enum"):
                                            info_type = "int" if isinstance(real_prop, ENUM_TYPES) else prop_type
                                            self._properties.append((prop_name, info_type))
        except (AssertionError, RuntimeError):
            pass


class LomIntrospection:
    def __init__(self, directory, exclude=None, *a, **k):
        if exclude is None:
            exclude = []
        (super().__init__)(*a, **k)
        self._lom_classes = []
        self._lom_modules = []
        self._excluded = exclude
        self._create_introspection_for_dir(directory)

    @property
    def lom_classes(self):
        return self._lom_classes

    def _is_relevant_class(self, class_object):
        return (
            is_class(class_object)
            and old_hasattr(class_object, "__module__")
            and sys.modules.get(class_object.__module__) in self._lom_modules
            and class_object not in self._lom_classes
            and class_object not in self._excluded
        )

    def _process_class(self, class_object):
        processed = False
        if self._is_relevant_class(class_object):
            self._lom_classes.append(class_object)
            processed = True
        return processed

    def _is_relevant_module(self, module_object):
        return isinstance(module_object, types.ModuleType) and module_object not in self._lom_modules

    def _process_module(self, module_object):
        processed = False
        if self._is_relevant_module(module_object):
            self._lom_modules.append(module_object)
            processed = True
        return processed

    def _create_introspection_for_module_or_class(self, attribute):
        self._process_class(attribute)
        for sub_attr_name in dir(attribute):
            try:
                sub_attribute = getattr(attribute, sub_attr_name)
                if self._process_class(sub_attribute):
                    for sub_sub_attr_name in dir(sub_attribute):
                        with contextlib.suppress(Exception):
                            self._process_class(getattr(sub_attribute, sub_sub_attr_name))

            except:
                pass

    def _create_introspection_for_dir(self, directory):
        for attr_name in list(dir(directory)):
            try:
                attribute = getattr(directory, attr_name)
                if attribute not in self._excluded:
                    if self._process_module(attribute) or is_class(attribute):
                        self._create_introspection_for_module_or_class(attribute)
            except:
                pass


class TopLevelControlSurfaceListParent:
    @property
    def control_surfaces(self):
        return get_control_surfaces()


_control_surface_list_parent = TopLevelControlSurfaceListParent()


def is_control_surfaces_list(path_component):
    return path_component in ("cs", "control_surfaces")


def wrap_control_surfaces_list(parent, cs_wrapper_registry):
    global _control_surface_list_parent
    return TupleWrapper.get_tuple_wrapper(
        (parent if parent is not None else _control_surface_list_parent),
        "control_surfaces",
        element_filter=is_real_control_surface,
        element_transform=(cs_wrapper_registry.wrap),
    )


class LomPathCalculator:
    def __init__(self, lom_object, external_device, *a, **k):
        (super().__init__)(*a, **k)
        self._path_components = self._calculate_path(lom_object, external_device)

    @property
    def path_components(self):
        return self._path_components

    def _find_root_object_path(self, external_device, lom_object):
        component = None
        for key in ROOT_KEYS:
            root_prop = get_root_prop(external_device, key)
            if not is_object_iterable(root_prop):
                if lom_object == root_prop:
                    component = key
                    break
            else:
                if lom_object in root_prop:
                    index = list(root_prop).index(lom_object)
                    component = "%s %d" % (key, index)
                    break

        return component

    def _find_property_object_path(self, lom_object, parent):
        component = None
        for key in list(PROPERTY_TYPES.keys()):
            if isinstance(lom_object, PROPERTY_TYPES[key]):
                if old_hasattr(parent, key):
                    if lom_object == getattr(parent, key):
                        component = key
                        break

        return component

    def _find_tuple_element_object_path(self, lom_object, parent):
        component = None
        for key in sorted(TUPLE_TYPES.keys()):
            if old_hasattr(parent, key):
                property = getattr(parent, key)
                if lom_object in property:
                    index = list(property).index(lom_object)
                    component = "%s %d" % (key, index)
                    break

        return component

    def _prepend_path_component(self, component, components):
        if component is not None:
            return [component, *components]
        return []

    def _calculate_path(self, lom_object, external_device_getter):
        components = []
        while lom_object is not None:
            component = None
            parent = lom_object.canonical_parent
            if parent is not None:
                component = self._find_property_object_path(lom_object, parent) or self._find_tuple_element_object_path(
                    lom_object,
                    parent,
                )
                components = self._prepend_path_component(component, components)
                if components == []:
                    break
            else:
                component = self._find_root_object_path(external_device_getter, lom_object)
                components = self._prepend_path_component(component, components)
                break
            lom_object = parent

        return components


class LomPathResolver:
    def __init__(self, path_components, external_device, lom_classes, list_manager, cs_wrapper_registry=None, *a, **k):
        (super().__init__)(*a, **k)
        self._external_device = external_device
        self._lom_classes = lom_classes
        self._list_manager = list_manager
        self._cs_wrapper_registry = cs_wrapper_registry
        self._lom_object = self._calculate_object_from_path(path_components)

    @property
    def lom_object(self):
        return self._lom_object

    def _tuple_element_from_path(self, path_components):
        lom_object = None
        parent = None
        attribute = path_components[-1]
        if len(path_components) > 1:
            parent = self._calculate_object_from_path(path_components[:-1])
        if is_control_surfaces_list(attribute):
            lom_object = wrap_control_surfaces_list(parent, self._cs_wrapper_registry)
        else:
            if parent is not None:
                if old_hasattr(parent, attribute):
                    selector = (
                        self._list_manager.get_list_wrapper
                        if is_cplusplus_lom_object(parent)
                        else TupleWrapper.get_tuple_wrapper
                    )
                    lom_object = selector(parent, attribute)
        return lom_object

    def _property_object_from_path(self, path_components):
        prev_component = path_components[0]
        lom_object = get_root_prop(self._external_device, path_components[0])
        if prev_component == THIS_DEVICE:
            if not liveobj_valid(lom_object):
                raise LomObjectError("The this_device root is only available inside a Max for Live device")
            components = [lom_object]
            for component in path_components[1:]:
                try:
                    if component.isdigit():
                        index = int(component)
                        if is_control_surfaces_list(prev_component):
                            parent = components[-2] if len(components) > 1 else None
                            tuple_wrapper = wrap_control_surfaces_list(parent, self._cs_wrapper_registry)
                            lom_object = tuple_wrapper.get_list()[index]
                        else:
                            lom_object = lom_object[index]
                    else:
                        lom_object = getattr(lom_object, component)
                    components.append(lom_object)
                except IndexError:
                    raise LomAttributeError(f"invalid index of component '{prev_component}'")
                except AttributeError:
                    raise LomAttributeError(f"invalid path component '{component}'")
                else:
                    prev_component = component

            if not is_lom_object(lom_object, self._lom_classes):
                raise LomObjectError(f"component '{prev_component}' is not an object")
            return lom_object
        return None

    def _calculate_object_from_path(self, path_components):
        lom_object = None
        if len(path_components) > 0:
            selector = (
                self._tuple_element_from_path
                if path_components[-1] in list(TUPLE_TYPES.keys())
                else self._property_object_from_path
            )
            lom_object = selector(path_components)
        return lom_object



# decompyle3 version 3.9.0
# Python bytecode version base 3.7.0 (3394)
# Decompiled from: Python 3.7.16 (default, Jan 17 2023, 09:28:58)
# [Clang 14.0.6 ]
# Embedded file name: output/Live/mac_universal_64_static/Release/python-bundle/MIDI Remote Scripts/ableton/v3/control_surface/components/__init__.py
# Compiled at: 2023-11-21 10:21:18
# Size of source mod 2**32: 3998 bytes
from .accent import AccentComponent
from .active_parameter import ActiveParameterComponent
from .auto_arm import AutoArmComponent
from .background import BackgroundComponent, ModifierBackgroundComponent, TranslatingBackgroundComponent
from .channel_strip import ChannelStripComponent
from .clip_actions import ClipActionsComponent
from .clip_slot import ClipSlotComponent
from .clipboard import ClipboardComponent
from .device import DeviceComponent
from .device_bank_navigation import DeviceBankNavigationComponent
from .device_parameters import DeviceParametersComponent
from .drum_group import DEFAULT_DRUM_TRANSLATION_CHANNEL, DrumGroupComponent, DrumPadClipboardComponent
from .drum_group_scroll import DrumGroupScrollComponent
from .grid_resolution import GRID_RESOLUTIONS, GridResolutionComponent
from .loop_selector import LoopSelectorComponent
from .mixer import MixerComponent, SendIndexManager
from .note_editor import DEFAULT_STEP_TRANSLATION_CHANNEL, NoteEditorComponent, PitchProvider
from .page import Pageable, PageComponent
from .paginator import NoteEditorPaginator, Paginator
from .playable import PlayableComponent
from .playhead import PlayheadComponent
from .recording import (
    BasicRecordingMethod,
    NextSlotRecordingMethod,
    NextSlotWithOverdubRecordingMethod,
    RecordingComponent,
    RecordingMethod,
    ViewBasedRecordingComponent,
)
from .scene import SceneComponent
from .scroll import Scrollable, ScrollComponent
from .session import ClipSlotClipboardComponent, SessionComponent
from .session_navigation import SessionNavigationComponent
from .session_overview import SessionOverviewComponent
from .session_ring import SessionRingComponent
from .simple_device_navigation import SimpleDeviceNavigationComponent
from .sliced_simpler import DEFAULT_SIMPLER_TRANSLATION_CHANNEL, SlicedSimplerComponent
from .step_sequence import SequencerClip, StepSequenceComponent, create_sequencer_clip
from .target_track import ArmedTargetTrackComponent, TargetTrackComponent
from .transport import TransportComponent
from .undo_redo import UndoRedoComponent
from .view_control import ViewControlComponent
from .view_toggle import ViewToggleComponent

__all__ = (
    "DEFAULT_DRUM_TRANSLATION_CHANNEL",
    "DEFAULT_SIMPLER_TRANSLATION_CHANNEL",
    "DEFAULT_STEP_TRANSLATION_CHANNEL",
    "GRID_RESOLUTIONS",
    "AccentComponent",
    "ActiveParameterComponent",
    "ArmedTargetTrackComponent",
    "AutoArmComponent",
    "BackgroundComponent",
    "BasicRecordingMethod",
    "ChannelStripComponent",
    "ClipActionsComponent",
    "ClipboardComponent",
    "ClipSlotClipboardComponent",
    "ClipSlotComponent",
    "DeviceBankNavigationComponent",
    "DeviceComponent",
    "DeviceParametersComponent",
    "DrumGroupComponent",
    "DrumGroupScrollComponent",
    "DrumPadClipboardComponent",
    "GridResolutionComponent",
    "LoopSelectorComponent",
    "MixerComponent",
    "ModifierBackgroundComponent",
    "NextSlotRecordingMethod",
    "NextSlotWithOverdubRecordingMethod",
    "NoteEditorComponent",
    "NoteEditorPaginator",
    "Pageable",
    "PageComponent",
    "Paginator",
    "PitchProvider",
    "PlayableComponent",
    "PlayheadComponent",
    "RecordingComponent",
    "RecordingMethod",
    "SceneComponent",
    "Scrollable",
    "ScrollComponent",
    "SendIndexManager",
    "SequencerClip",
    "SessionComponent",
    "SessionNavigationComponent",
    "SessionOverviewComponent",
    "SessionRingComponent",
    "SimpleDeviceNavigationComponent",
    "SlicedSimplerComponent",
    "StepSequenceComponent",
    "TargetTrackComponent",
    "TranslatingBackgroundComponent",
    "TransportComponent",
    "UndoRedoComponent",
    "ViewBasedRecordingComponent",
    "ViewControlComponent",
    "ViewToggleComponent",
    "create_sequencer_clip",
)



# decompyle3 version 3.9.0
# Python bytecode version base 3.7.0 (3394)
# Decompiled from: Python 3.7.16 (default, Jan 17 2023, 09:28:58)
# [Clang 14.0.6 ]
# Embedded file name: output/Live/mac_universal_64_static/Release/python-bundle/MIDI Remote Scripts/ableton/v2/control_surface/components/device.py
# Compiled at: 2023-11-21 10:21:18
# Size of source mod 2**32: 5798 bytes
from control_surface.simpler_slice_nudging import SimplerSliceNudging

from ...base import depends, listens, liveobj_changed, liveobj_valid
from ...control_surface import Component, ParameterProvider, create_device_bank


class DeviceComponent(ParameterProvider, Component):
    __events__ = ("device",)
    _provided_parameters = ()

    @depends(device_provider=None)
    def __init__(
        self,
        device_decorator_factory=None,
        banking_info=None,
        device_bank_registry=None,
        device_provider=None,
        decoupled_parameter_list_change_notifications=False,
        *a,
        **k,
    ):
        self._bank = None
        self._banking_info = banking_info
        self._decorated_device = None
        self._decorator_factory = device_decorator_factory
        self._device_provider = device_provider
        self._device_bank_registry = device_bank_registry
        self._parameters_dirty = True
        self._decoupled_parameter_list_change_notifications = decoupled_parameter_list_change_notifications
        (super().__init__)(*a, **k)
        self._initialize_subcomponents()
        self._DeviceComponent__on_provided_device_changed.subject = device_provider
        self._DeviceComponent__on_provided_device_changed()

    def set_device(self, device):
        self._device_provider.device = device

    def device(self):
        return self._decorated_device

    def _initialize_subcomponents(self):
        self._slice_nudging = self.register_disconnectable(SimplerSliceNudging())

    @property
    def parameters(self):
        return self._provided_parameters

    @listens("device_bank")
    def __on_bank_changed(self, device, bank):
        if device == self.device():
            self._set_bank_index(bank)

    def _set_bank_index(self, bank):
        if self._bank is not None:
            self._bank.index = bank

    def _update_parameters(self):
        self._parameters_dirty = True
        if not self._decoupled_parameter_list_change_notifications:
            self.update_and_notify_parameters()

    def update_and_notify_parameters(self):
        if self._parameters_dirty:
            self._provided_parameters = self._get_provided_parameters()
            self.notify_parameters()
            self._parameters_dirty = False

    def _setup_bank(self, device, bank_factory=create_device_bank):
        if self._bank is not None:
            self.disconnect_disconnectable(self._bank)
            self._bank = None
        if liveobj_valid(device):
            self._bank = self.register_disconnectable(bank_factory(device, self._banking_info))

    def _get_decorated_device(self, device):
        if self._decorator_factory is not None:
            return self._decorator_factory.decorate(device)
        return device

    def _device_changed(self, device):
        current_device = getattr(self.device(), "_live_object", self.device())
        return liveobj_changed(current_device, device)

    @listens("device")
    def __on_provided_device_changed(self):
        self._on_device_changed(self._device_provider.device)

    @listens("parameters")
    def __on_parameters_changed_in_device(self):
        self._update_parameters()

    def _on_device_changed(self, device):
        if self._device_changed(device):
            self._set_device(device)

    def _set_decorated_device(self, decorated_device):
        self._set_decorated_device_for_subcomponents(decorated_device)
        self._setup_bank(decorated_device)
        self._on_bank_parameters_changed.subject = self._bank
        self._decorated_device = decorated_device

    def _set_device(self, device):
        self._set_device_for_subcomponents(device)
        decorated_device = self._get_decorated_device(device)
        self._set_decorated_device(decorated_device)
        device_bank_registry = self._device_bank_registry
        self._DeviceComponent__on_bank_changed.subject = device_bank_registry
        bank_index_for_device = device_bank_registry.get_device_bank(device)
        self._set_bank_index(bank_index_for_device)
        self.notify_device()
        self._update_parameters()
        self._DeviceComponent__on_parameters_changed_in_device.subject = device

    def _set_device_for_subcomponents(self, device):
        pass

    def _set_decorated_device_for_subcomponents(self, decorated_device):
        self._slice_nudging.set_device(decorated_device)

    @listens("parameters")
    def _on_bank_parameters_changed(self):
        self._update_parameters()

    def _current_bank_details(self):
        if self._bank is not None:
            return (self._bank.name, self._bank.parameters)
        return ("", [None] * 8)

    def _number_of_parameter_banks(self):
        if self._bank is not None:
            return self._bank.bank_count()
        return 0

    def _get_provided_parameters(self):
        _, parameters = self._current_bank_details() if self.device() else (None, ())
        return [self._create_parameter_info(param, name) for param, name in parameters]

    def _create_parameter_info(self, parameter, name):
        raise NotImplementedError()



from __future__ import annotations

from typing import Any, Callable

from Live import *
from Live.Application import *
from Live.Base import *
from Live.Browser import *
from Live.CcControlDevice import *
from Live.Chain import *
from Live.ChainMixerDevice import *
from Live.Clip import *
from Live.ClipSlot import *
from Live.CompressorDevice import *
from Live.Conversions import *
from Live.Device import *
from Live.DeviceIO import *
from Live.DeviceParameter import *
from Live.DriftDevice import *
from Live.DrumChain import *
from Live.DrumPad import *
from Live.Eq8Device import *
from Live.Groove import *
from Live.GroovePool import *
from Live.HybridReverbDevice import *
from Live.Listener import *
from Live.LomObject import *
from Live.MaxDevice import *
from Live.MeldDevice import *
from Live.MidiMap import *
from Live.MixerDevice import *
from Live.PluginDevice import *
from Live.RackDevice import *
from Live.RoarDevice import *
from Live.Sample import *
from Live.ShifterDevice import *
from Live.SimplerDevice import *
from Live.Song import *
from Live.SpectralResonatorDevice import *
from Live.Track import *
from Live.WavetableDevice import *


class Scene:
    """
    This class represents an series of ClipSlots in Lives Sessionview matrix.
    """

    def add_clip_slots_listener(self, listener: Callable) -> None:
        """
        add_clip_slots_listener( (Scene)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "clip_slots" has changed.

            C++ signature :
                void add_clip_slots_listener(TPyHandle<AScene>,boost::python::api::object)
        """

    def add_color_index_listener(self, listener: Callable) -> None:
        """
        add_color_index_listener( (Scene)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "color_index" has changed.

            C++ signature :
                void add_color_index_listener(TPyHandle<AScene>,boost::python::api::object)
        """

    def add_color_listener(self, listener: Callable) -> None:
        """
        add_color_listener( (Scene)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "color" has changed.

            C++ signature :
                void add_color_listener(TPyHandle<AScene>,boost::python::api::object)
        """

    def add_is_triggered_listener(self, listener: Callable) -> None:
        """
        add_is_triggered_listener( (Scene)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "is_triggered" has changed.

            C++ signature :
                void add_is_triggered_listener(TPyHandle<AScene>,boost::python::api::object)
        """

    def add_name_listener(self, listener: Callable) -> None:
        """
        add_name_listener( (Scene)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "name" has changed.

            C++ signature :
                void add_name_listener(TPyHandle<AScene>,boost::python::api::object)
        """

    def add_tempo_enabled_listener(self, listener: Callable) -> None:
        """
        add_tempo_enabled_listener( (Scene)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "tempo_enabled" has changed.

            C++ signature :
                void add_tempo_enabled_listener(TPyHandle<AScene>,boost::python::api::object)
        """

    def add_tempo_listener(self, listener: Callable) -> None:
        """
        add_tempo_listener( (Scene)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "tempo" has changed.

            C++ signature :
                void add_tempo_listener(TPyHandle<AScene>,boost::python::api::object)
        """

    def add_time_signature_denominator_listener(self, listener: Callable) -> None:
        """
        add_time_signature_denominator_listener( (Scene)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "time_signature_denominator" has changed.

            C++ signature :
                void add_time_signature_denominator_listener(TPyHandle<AScene>,boost::python::api::object)
        """

    def add_time_signature_enabled_listener(self, listener: Callable) -> None:
        """
        add_time_signature_enabled_listener( (Scene)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "time_signature_enabled" has changed.

            C++ signature :
                void add_time_signature_enabled_listener(TPyHandle<AScene>,boost::python::api::object)
        """

    def add_time_signature_numerator_listener(self, listener: Callable) -> None:
        """
        add_time_signature_numerator_listener( (Scene)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "time_signature_numerator" has changed.

            C++ signature :
                void add_time_signature_numerator_listener(TPyHandle<AScene>,boost::python::api::object)
        """

    def clip_slots_has_listener(self, listener: Callable) -> bool:
        """
        clip_slots_has_listener( (Scene)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "clip_slots".

            C++ signature :
                bool clip_slots_has_listener(TPyHandle<AScene>,boost::python::api::object)
        """

    def color_has_listener(self, listener: Callable) -> bool:
        """
        color_has_listener( (Scene)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "color".

            C++ signature :
                bool color_has_listener(TPyHandle<AScene>,boost::python::api::object)
        """

    def color_index_has_listener(self, listener: Callable) -> bool:
        """
        color_index_has_listener( (Scene)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "color_index".

            C++ signature :
                bool color_index_has_listener(TPyHandle<AScene>,boost::python::api::object)
        """

    def fire(self, force_legato: bool = False, can_select_scene_on_launch: bool = True) -> None:
        """
        fire( (Scene)arg1 [, (bool)force_legato=False [, (bool)can_select_scene_on_launch=True]]) -> None :
            Fire the scene directly. Will fire all clipslots that this scene owns and
            select the scene itself.

            C++ signature :
                void fire(TPyHandle<AScene> [,bool=False [,bool=True]])
        """

    def fire_as_selected(self, force_legato: bool = False) -> None:
        """
        fire_as_selected( (Scene)arg1 [, (bool)force_legato=False]) -> None :
            Fire the selected scene. Will fire all clipslots that this scene owns and
            select the next scene if necessary.

            C++ signature :
                void fire_as_selected(TPyHandle<AScene> [,bool=False])
        """

    def is_triggered_has_listener(self, listener: Callable) -> bool:
        """
        is_triggered_has_listener( (Scene)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "is_triggered".

            C++ signature :
                bool is_triggered_has_listener(TPyHandle<AScene>,boost::python::api::object)
        """

    def name_has_listener(self, listener: Callable) -> bool:
        """
        name_has_listener( (Scene)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "name".

            C++ signature :
                bool name_has_listener(TPyHandle<AScene>,boost::python::api::object)
        """

    def remove_clip_slots_listener(self, listener: Callable) -> None:
        """
        remove_clip_slots_listener( (Scene)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "clip_slots".

            C++ signature :
                void remove_clip_slots_listener(TPyHandle<AScene>,boost::python::api::object)
        """

    def remove_color_index_listener(self, listener: Callable) -> None:
        """
        remove_color_index_listener( (Scene)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "color_index".

            C++ signature :
                void remove_color_index_listener(TPyHandle<AScene>,boost::python::api::object)
        """

    def remove_color_listener(self, listener: Callable) -> None:
        """
        remove_color_listener( (Scene)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "color".

            C++ signature :
                void remove_color_listener(TPyHandle<AScene>,boost::python::api::object)
        """

    def remove_is_triggered_listener(self, listener: Callable) -> None:
        """
        remove_is_triggered_listener( (Scene)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "is_triggered".

            C++ signature :
                void remove_is_triggered_listener(TPyHandle<AScene>,boost::python::api::object)
        """

    def remove_name_listener(self, listener: Callable) -> None:
        """
        remove_name_listener( (Scene)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "name".

            C++ signature :
                void remove_name_listener(TPyHandle<AScene>,boost::python::api::object)
        """

    def remove_tempo_enabled_listener(self, listener: Callable) -> None:
        """
        remove_tempo_enabled_listener( (Scene)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "tempo_enabled".

            C++ signature :
                void remove_tempo_enabled_listener(TPyHandle<AScene>,boost::python::api::object)
        """

    def remove_tempo_listener(self, listener: Callable) -> None:
        """
        remove_tempo_listener( (Scene)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "tempo".

            C++ signature :
                void remove_tempo_listener(TPyHandle<AScene>,boost::python::api::object)
        """

    def remove_time_signature_denominator_listener(self, listener: Callable) -> None:
        """
        remove_time_signature_denominator_listener( (Scene)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "time_signature_denominator".

            C++ signature :
                void remove_time_signature_denominator_listener(TPyHandle<AScene>,boost::python::api::object)
        """

    def remove_time_signature_enabled_listener(self, listener: Callable) -> None:
        """
        remove_time_signature_enabled_listener( (Scene)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "time_signature_enabled".

            C++ signature :
                void remove_time_signature_enabled_listener(TPyHandle<AScene>,boost::python::api::object)
        """

    def remove_time_signature_numerator_listener(self, listener: Callable) -> None:
        """
        remove_time_signature_numerator_listener( (Scene)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "time_signature_numerator".

            C++ signature :
                void remove_time_signature_numerator_listener(TPyHandle<AScene>,boost::python::api::object)
        """

    def set_fire_button_state(self, arg2: bool) -> None:
        """
        set_fire_button_state( (Scene)arg1, (bool)arg2) -> None :
            Set the scene's fire button state directly. Supports all launch modes.

            C++ signature :
                void set_fire_button_state(TPyHandle<AScene>,bool)
        """

    def tempo_enabled_has_listener(self, listener: Callable) -> bool:
        """
        tempo_enabled_has_listener( (Scene)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "tempo_enabled".

            C++ signature :
                bool tempo_enabled_has_listener(TPyHandle<AScene>,boost::python::api::object)
        """

    def tempo_has_listener(self, listener: Callable) -> bool:
        """
        tempo_has_listener( (Scene)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "tempo".

            C++ signature :
                bool tempo_has_listener(TPyHandle<AScene>,boost::python::api::object)
        """

    def time_signature_denominator_has_listener(self, listener: Callable) -> bool:
        """
        time_signature_denominator_has_listener( (Scene)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "time_signature_denominator".

            C++ signature :
                bool time_signature_denominator_has_listener(TPyHandle<AScene>,boost::python::api::object)
        """

    def time_signature_enabled_has_listener(self, listener: Callable) -> bool:
        """
        time_signature_enabled_has_listener( (Scene)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "time_signature_enabled".

            C++ signature :
                bool time_signature_enabled_has_listener(TPyHandle<AScene>,boost::python::api::object)
        """

    def time_signature_numerator_has_listener(self, listener: Callable) -> bool:
        """
        time_signature_numerator_has_listener( (Scene)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "time_signature_numerator".

            C++ signature :
                bool time_signature_numerator_has_listener(TPyHandle<AScene>,boost::python::api::object)
        """

    @property
    def canonical_parent(self) -> Any:
        """
        Get the canonical parent of the scene.
        """

    @property
    def clip_slots(self) -> list[ClipSlot]:
        """
        return a list of clipslots (see class AClipSlot) that this scene covers.
        """

    @property
    def color(self) -> Any:
        """
        Get/set access to the color of the scene (RGB).
        """

    @color.setter
    def color(self, value: Any) -> None:
        pass

    @property
    def color_index(self) -> Any:
        """
        Get/set access to the color index of the scene. Can be None for no color.
        """

    @color_index.setter
    def color_index(self, value: Any) -> None:
        pass

    @property
    def is_empty(self) -> bool:
        """
        Returns True if all clip slots of this scene are empty.
        """

    @property
    def is_triggered(self) -> bool:
        """
        Const access to the scene's trigger state.
        """

    @property
    def name(self) -> str:
        """
        Get/Set the name of the scene.
        """

    @name.setter
    def name(self, value: str) -> None:
        pass

    @property
    def tempo(self) -> float:
        """
        Get/Set the tempo value of the scene.
        The song will use the scene's tempo as soon as the scene is fired.
        Returns -1 if the scene has no tempo property.
        """

    @tempo.setter
    def tempo(self, value: float) -> None:
        pass

    @property
    def tempo_enabled(self) -> Any:
        """
        Get/Set the active state of the scene tempo.
        When disabled, the scene will use the song's tempo,and the tempo value returned will be -1Returns a bool indicating the state of the scene's tempo
        """

    @tempo_enabled.setter
    def tempo_enabled(self, value: Any) -> None:
        pass

    @property
    def time_signature_denominator(self) -> Any:
        """
        Get/Set the scene's time signature denominator.
        The song will use the scene's time signature as soon as the scene is fired.
        Returns -1 if the scene has no time signature property.
        """

    @time_signature_denominator.setter
    def time_signature_denominator(self, value: Any) -> None:
        pass

    @property
    def time_signature_enabled(self) -> Any:
        """
        Get the active state of the scene time signature.
        When disabled, the scene will use the song's time signature,and the time signature values returned will be -1Returns a bool indicating the state of the scene's time signature
        """

    @property
    def time_signature_numerator(self) -> Any:
        """
        Get/Set the scene's time signature numerator.
        The song will use the scene's time signature as soon as the scene is fired.
        Returns -1 if the scene has no time signature property.
        """

    @time_signature_numerator.setter
    def time_signature_numerator(self, value: Any) -> None:
        pass



# decompyle3 version 3.9.0
# Python bytecode version base 3.7.0 (3394)
# Decompiled from: Python 3.7.16 (default, Jan 17 2023, 09:28:58)
# [Clang 14.0.6 ]
# Embedded file name: output/Live/mac_universal_64_static/Release/python-bundle/MIDI Remote Scripts/ableton/v3/control_surface/display/notifications/util.py
# Compiled at: 2023-11-21 10:21:18
# Size of source mod 2**32: 597 bytes
from __future__ import annotations

from typing import TYPE_CHECKING, Callable

if TYPE_CHECKING:
    from typing_extensions import LiteralString
else:
    LiteralString = str


def toggle_text_generator(format_string: LiteralString) -> Callable[[bool], str]:
    def notification_fn(is_on):
        return format_string.format("on" if is_on else "off")

    return notification_fn



# decompyle3 version 3.9.0
# Python bytecode version base 3.7.0 (3394)
# Decompiled from: Python 3.7.16 (default, Jan 17 2023, 09:28:58)
# [Clang 14.0.6 ]
# Embedded file name: output/Live/mac_universal_64_static/Release/python-bundle/MIDI Remote Scripts/ableton/v3/control_surface/components/device_parameters.py
# Compiled at: 2023-11-21 10:21:18
# Size of source mod 2**32: 2329 bytes
from itertools import zip_longest

from ...base import listens
from .. import Component, ParameterProvider
from ..controls import MappedSensitivitySettingControl, control_list


class DeviceParametersComponent(Component):
    controls = control_list(MappedSensitivitySettingControl, 8)

    def __init__(self, parameter_provider=None, name="Device_Parameters", *a, **k):
        (super().__init__)(a, name=name, **k)
        self.parameter_provider = parameter_provider

    @property
    def parameter_provider(self):
        return self._parameter_provider

    @parameter_provider.setter
    def parameter_provider(self, provider):
        self._parameter_provider = provider or ParameterProvider()
        self._DeviceParametersComponent__on_parameters_changed.subject = self._parameter_provider
        self._update_parameters()

    def set_parameter_controls(self, encoders):
        if encoders:
            if len(encoders) > self.controls.control_count:
                self.controls.control_count = len(encoders)
        self.controls.set_control_element(encoders)
        self._connect_parameters()

    def _connect_parameters(self):
        parameters = self._parameter_provider.parameters[: self.controls.control_count]
        for control, parameter_info in zip_longest(self.controls, parameters):
            parameter = parameter_info.parameter if parameter_info else None
            control.mapped_parameter = parameter
            if parameter:
                control.update_sensitivities(
                    parameter_info.default_encoder_sensitivity,
                    parameter_info.fine_grain_encoder_sensitivity,
                )

    def update(self):
        super().update()
        self._update_parameters()

    def _update_parameters(self):
        if self.is_enabled():
            self._connect_parameters()

    @listens("parameters")
    def __on_parameters_changed(self):
        self._update_parameters()



from __future__ import annotations

from typing import Any, Callable

from Live import *
from Live.Application import *
from Live.Base import *
from Live.Browser import *
from Live.CcControlDevice import *
from Live.Chain import *
from Live.Clip import *
from Live.ClipSlot import *
from Live.CompressorDevice import *
from Live.Conversions import *
from Live.Device import *
from Live.DeviceIO import *
from Live.DeviceParameter import *
from Live.DriftDevice import *
from Live.DrumChain import *
from Live.DrumPad import *
from Live.Eq8Device import *
from Live.Groove import *
from Live.GroovePool import *
from Live.HybridReverbDevice import *
from Live.Listener import *
from Live.LomObject import *
from Live.MaxDevice import *
from Live.MeldDevice import *
from Live.MidiMap import *
from Live.MixerDevice import *
from Live.PluginDevice import *
from Live.RackDevice import *
from Live.RoarDevice import *
from Live.Sample import *
from Live.Scene import *
from Live.ShifterDevice import *
from Live.SimplerDevice import *
from Live.Song import *
from Live.SpectralResonatorDevice import *
from Live.Track import *
from Live.WavetableDevice import *


class ChainMixerDevice:
    """
    This class represents a Chain's Mixer Device in Live, which gives you
    access to the Volume, Panning, and Send properties of a Chain.
    """

    def add_sends_listener(self, listener: Callable) -> None:
        """
        add_sends_listener( (ChainMixerDevice)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "sends" has changed.

            C++ signature :
                void add_sends_listener(TPyHandle<ABranchMixerDevice>,boost::python::api::object)
        """

    def remove_sends_listener(self, listener: Callable) -> None:
        """
        remove_sends_listener( (ChainMixerDevice)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "sends".

            C++ signature :
                void remove_sends_listener(TPyHandle<ABranchMixerDevice>,boost::python::api::object)
        """

    def sends_has_listener(self, listener: Callable) -> bool:
        """
        sends_has_listener( (ChainMixerDevice)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "sends".

            C++ signature :
                bool sends_has_listener(TPyHandle<ABranchMixerDevice>,boost::python::api::object)
        """

    @property
    def canonical_parent(self) -> Any:
        """
        Get the canonical parent of the mixer device.
        """

    @property
    def chain_activator(self) -> Any:
        """
        Const access to the Chain's Activator Device Parameter.
        """

    @property
    def panning(self) -> Any:
        """
        Const access to the Chain's Panning Device Parameter.
        """

    @property
    def sends(self) -> Any:
        """
        Const access to the Chain's list of Send Amount Device Parameters.
        """

    @property
    def volume(self) -> Any:
        """
        Const access to the Chain's Volume Device Parameter.
        """



# decompyle3 version 3.9.0
# Python bytecode version base 3.7.0 (3394)
# Decompiled from: Python 3.7.16 (default, Jan 17 2023, 09:28:58)
# [Clang 14.0.6 ]
# Embedded file name: output/Live/mac_universal_64_static/Release/python-bundle/MIDI Remote Scripts/_Framework/CompoundComponent.py
# Compiled at: 2023-11-21 10:21:18
# Size of source mod 2**32: 2000 bytes
from .ControlSurfaceComponent import ControlSurfaceComponent


class CompoundComponent(ControlSurfaceComponent):
    def __init__(self, *a, **k):
        (super().__init__)(*a, **k)
        self._sub_components = []

    def update_all(self):
        self.update()
        for component in self._sub_components:
            component.update_all()

    def register_component(self, component):
        component._set_enabled_recursive(self.is_enabled())
        self._sub_components.append(component)
        return component

    def register_components(self, *a):
        return list(map(self.register_component, a))

    def has_component(self, component):
        return component in self._sub_components

    def set_enabled(self, enable):
        super().set_enabled(enable)
        for component in self._sub_components:
            component._set_enabled_recursive(self.is_enabled())

    def _set_enabled_recursive(self, enable):
        super()._set_enabled_recursive(enable)
        for component in self._sub_components:
            component._set_enabled_recursive(self.is_enabled())

    def set_allow_update(self, allow_updates):
        allow = bool(allow_updates)
        if self._allow_updates != allow:
            for component in self._sub_components:
                component.set_allow_update(allow)

            super().set_allow_update(allow)



# decompyle3 version 3.9.0
# Python bytecode version base 3.7.0 (3394)
# Decompiled from: Python 3.7.16 (default, Jan 17 2023, 09:28:58)
# [Clang 14.0.6 ]
# Embedded file name: output/Live/mac_universal_64_static/Release/python-bundle/MIDI Remote Scripts/ableton/v2/control_surface/control/encoder.py
# Compiled at: 2023-11-21 10:21:18
# Size of source mod 2**32: 13533 bytes
import logging

from ...base import clamp, const, lazy_attribute, old_hasattr, sign, task
from .control import Connectable, InputControl, SendValueMixin, control_event

logger = logging.getLogger(__name__)
__all__ = ("EncoderControl", "StepEncoderControl", "ListValueEncoderControl", "ListIndexEncoderControl", "ValueStepper")


class EncoderControl(InputControl):
    TOUCH_TIME = 0.5
    touched = control_event("touched")
    released = control_event("released")

    class State(InputControl.State, Connectable):
        def __init__(self, control=None, manager=None, touch_event_delay=0, *a, **k):
            (super(EncoderControl.State, self).__init__)(a, control=control, manager=manager, **k)
            self._is_touched = False
            self._touch_value_slot = None
            self._timer_based = False
            self._touch_event_delay = touch_event_delay
            self._touch_value_slot = self.register_slot(None, self._on_touch_value, "value")

        @property
        def is_touched(self):
            return self._is_touched

        def set_control_element(self, control_element):
            if self._control_element != control_element or self._lost_touch_element(self._control_element):
                self._release_encoder()
                self._kill_all_tasks()
            super(EncoderControl.State, self).set_control_element(control_element)
            self._touch_value_slot.subject = (
                self._get_touch_element(control_element) if control_element is not None else None
            )
            if control_element:
                if old_hasattr(control_element, "is_pressed"):
                    if control_element.is_pressed():
                        self._touch_encoder()

        def _get_touch_element(self, control_element):
            return getattr(control_element, "touch_element", None)

        def _lost_touch_element(self, control_element):
            return control_element is not None and self._get_touch_element(control_element) is None

        def _touch_encoder(self):
            is_touched = self._is_touched
            self._is_touched = True
            if not is_touched:
                self._call_listener("touched")

        def _release_encoder(self):
            is_touched = self._is_touched
            self._is_touched = False
            if is_touched:
                self._call_listener("released")

        def _event_listener_required(self):
            return True

        def _notify_encoder_value(self, value, *a, **k):
            normalized_value = self._control_element.normalize_value(value)
            self._call_listener("value", normalized_value)
            self.connected_property_value = normalized_value

        def _on_value(self, value, *a, **k):
            if self._notifications_enabled():
                if not self._is_touched:
                    self._touch_encoder()
                    if self._delayed_touch_task.is_running:
                        self._delayed_touch_task.kill()
                    else:
                        self._timer_based = True
                        self._timer_based_release_task.restart()
                else:
                    if self._timer_based:
                        self._timer_based_release_task.restart()
                if self._control_element:
                    (self._notify_encoder_value)(value, *a, **k)

        def _on_touch_value(self, value, *a, **k):
            if self._notifications_enabled():
                if value:
                    if self._touch_event_delay > 0:
                        self._delayed_touch_task.restart()
                    else:
                        self._touch_encoder()
                else:
                    self._release_encoder()
                    self._delayed_touch_task.kill()
                self._cancel_timer_based_events()

        @lazy_attribute
        def _timer_based_release_task(self):
            return self.tasks.add(task.sequence(task.wait(EncoderControl.TOUCH_TIME), task.run(self._release_encoder)))

        @lazy_attribute
        def _delayed_touch_task(self):
            return self.tasks.add(task.sequence(task.wait(self._touch_event_delay), task.run(self._touch_encoder)))

        def _cancel_timer_based_events(self):
            if self._timer_based:
                self._timer_based_release_task.kill()
                self._timer_based = False

        def _kill_all_tasks(self):
            self._timer_based_release_task.kill()
            self._delayed_touch_task.kill()

    def __init__(self, *a, **k):
        super().__init__(extra_args=a, extra_kws=k)


class ValueStepper:
    def __init__(self, num_steps=10, *a, **k):
        (super().__init__)(*a, **k)
        self._step = 0.0
        self._num_steps = float(num_steps)

    @property
    def num_steps(self):
        return self._num_steps

    def advance(self, value):
        result = 0
        if sign(value) != sign(self._step):
            self.reset()
        new_value = self._step + value * self._num_steps
        if int(new_value) != int(self._step):
            result = int(new_value)
            self.reset()
        else:
            self._step = new_value
        return result

    def reset(self):
        self._step = 0.0


class StepEncoderControl(EncoderControl):
    class State(EncoderControl.State):
        def __init__(self, num_steps=10, *a, **k):
            (super(StepEncoderControl.State, self).__init__)(*a, **k)
            self._stepper = ValueStepper(num_steps=num_steps)

        def _notify_encoder_value(self, value, *a, **k):
            steps = self._stepper.advance(self._control_element.normalize_value(value))
            if steps != 0:
                self._call_listener("value", steps)
                self._on_stepped(steps)

        def _on_stepped(self, steps):
            self.connected_property_value = steps

        def _on_touch_value(self, value, *a, **k):
            (super(StepEncoderControl.State, self)._on_touch_value)(value, *a, **k)
            if not value:
                self._stepper.reset()

        @property
        def num_steps(self):
            return self._stepper._num_steps


class ListValueEncoderControl(StepEncoderControl):
    class State(StepEncoderControl.State):
        def __init__(self, *a, **k):
            (super(ListValueEncoderControl.State, self).__init__)(*a, **k)
            self._list_values_getter = None

        def connect_list_property(self, subject, list_property_name=None, current_value_property_name=None):
            self.connect_property(subject, current_value_property_name)
            self._list_values_getter = lambda: getattr(subject, list_property_name)

        def connect_static_list(self, subject, current_value_property_name=None, list_values=None):
            self.connect_property(subject, current_value_property_name)
            self._list_values_getter = const(list_values)

        def disconnect_property(self):
            super(ListValueEncoderControl.State, self).disconnect_property()
            self._list_values_getter = None

        def _on_stepped(self, steps):
            if self._list_values_getter is not None:
                list_values = self._list_values_getter()
                try:
                    current_index = list_values.index(self.connected_property_value)
                except ValueError:
                    logger.warning("Encoder was turned, but current value is not in list!")
                    current_index = 0

                new_index = clamp(current_index + steps, 0, len(list_values) - 1)
                self.connected_property_value = list_values[new_index]


class ListIndexEncoderControl(StepEncoderControl):
    class State(StepEncoderControl.State):
        def __init__(self, *a, **k):
            (super(ListIndexEncoderControl.State, self).__init__)(*a, **k)
            self._max_index = None

        def connect_list_property(self, subject, current_index_property_name=None, max_index=None):
            self.connect_property(subject, current_index_property_name)
            self._max_index = max_index

        def disconnect_property(self):
            super(ListIndexEncoderControl.State, self).disconnect_property()
            self._max_index = None

        def _on_stepped(self, steps):
            if self._max_index is not None:
                new_index = clamp(self.connected_property_value + steps, 0, self._max_index)
                self.connected_property_value = new_index


class SendValueEncoderControl(EncoderControl):
    class State(SendValueMixin, EncoderControl.State):
        pass



# decompyle3 version 3.9.0
# Python bytecode version base 3.7.0 (3394)
# Decompiled from: Python 3.7.16 (default, Jan 17 2023, 09:28:58)
# [Clang 14.0.6 ]
# Embedded file name: output/Live/mac_universal_64_static/Release/python-bundle/MIDI Remote Scripts/ableton/v3/control_surface/elements/encoder.py
# Compiled at: 2023-11-21 10:21:18
# Size of source mod 2**32: 10512 bytes
from ableton.v2.control_surface.elements import EncoderElement as EncoderElementBase
from ableton.v2.control_surface.elements.encoder import _map_modes

from ...base import lazy_attribute, listenable_property, listens, task
from ...live import display_name, liveobj_valid, parameter_value_to_midi_value
from .. import MIDI_CC_TYPE
from ..controls import is_internal_parameter
from ..display import Renderable
from ..parameter_mapping_sensitivities import DEFAULT_CONTINUOUS_PARAMETER_SENSITIVITY, FINE_GRAIN_SENSITIVITY_FACTOR


class MappingSensitivity:
    def __init__(self, standard, parent):
        fine_grain = standard * FINE_GRAIN_SENSITIVITY_FACTOR
        self._standard = standard
        self._fine_grain = fine_grain
        self._use_fine_grain = False
        self._parent = parent
        self._parent.set_sensitivities = self._set_sensitivities
        self._default_encoder_sensitivity = parent.encoder_sensitivity
        self.reset = lambda: self._set_sensitivities(standard, fine_grain)
        self._update_sensitivity()

    def should_use_fine_grain(self, use_fine_grain):
        self._use_fine_grain = use_fine_grain
        self._update_sensitivity()

    def _set_sensitivities(self, standard, fine_grain):
        self._standard = standard
        self._fine_grain = fine_grain
        self._update_sensitivity()

    def _update_sensitivity(self):
        sensitivity = self._fine_grain if self._use_fine_grain else self._standard
        self._parent.mapping_sensitivity = sensitivity
        if self._parent.is_mapped_to_parameter():
            self._parent.encoder_sensitivity = sensitivity
        else:
            self._parent.encoder_sensitivity = self._default_encoder_sensitivity


class EncoderElement(EncoderElementBase, Renderable):
    __events__ = ("parameter",)
    mapped_object = listenable_property.managed(None)

    def __init__(
        self,
        identifier,
        channel=0,
        msg_type=MIDI_CC_TYPE,
        map_mode=_map_modes.absolute,
        mapping_sensitivity=DEFAULT_CONTINUOUS_PARAMETER_SENSITIVITY,
        sensitivity_modifier=None,
        needs_takeover=True,
        is_feedback_enabled=False,
        feedback_delay=0,
        *a,
        **k,
    ):
        (super().__init__)(msg_type, channel, identifier, map_mode, *a, is_feedback_enabled=is_feedback_enabled, **k)
        self._needs_internal_parameter_feedback_delay = feedback_delay >= 0
        self._block_internal_parameter_feedback = False
        self._sensitivity = MappingSensitivity(mapping_sensitivity, self)
        self._sensitivity_modifier = sensitivity_modifier
        self._EncoderElement__on_sensitivity_modifier_value.subject = sensitivity_modifier
        self.set_needs_takeover(needs_takeover)
        self.set_feedback_delay(feedback_delay)
        self._feedback_values = self._mapping_feedback_values()
        self._update_listeners_task = self._tasks.add(task.run(self._update_parameter_listeners))
        self._update_listeners_task.kill()

    def disconnect(self):
        self.mapped_object = None
        super().disconnect()

    @property
    def parameter(self):
        if liveobj_valid(self._parameter_to_map_to):
            if not self.is_mapped_manually():
                return self._parameter_to_map_to
            return None
        return None

    @listenable_property
    def parameter_name(self):
        if self.is_mapped_to_parameter():
            return display_name(self.mapped_object)
        return ""

    @listenable_property
    def parameter_value(self):
        if self.mapped_object:
            return str(self.mapped_object)
        return ""

    def message_map_mode(self):
        return self._map_mode

    def reset_state(self):
        self._sensitivity.reset()
        super().reset_state()

    def is_mapped_to_parameter(self):
        return liveobj_valid(self.mapped_object) and not self.is_mapped_manually()

    def is_mapped_manually(self):
        return not self._is_mapped and not self._is_being_forwarded

    def install_connections(self, *a, **k):
        (super().install_connections)(*a, **k)
        if self.is_mapped_manually():
            self.mapped_object = None
        else:
            if self.mapped_object is None:
                self.reset()
        self._clear_parameter_listeners()
        self._update_listeners_task.restart()

    def connect_to(self, parameter):
        self.mapped_object = parameter
        if not is_internal_parameter(parameter):
            super().connect_to(parameter)
        self._clear_parameter_listeners()
        self._update_listeners_task.restart()

    def release_parameter(self):
        super().release_parameter()
        self.mapped_object = None
        self._update_listeners_task.restart()

    def receive_value(self, value):
        super().receive_value(value)
        if self._needs_internal_parameter_feedback_delay:
            self._block_internal_parameter_feedback = True
            self._unblock_internal_parameter_feedback_task.restart()

    def _parameter_value_changed(self):
        if is_internal_parameter(self.mapped_object):
            if not self._block_internal_parameter_feedback:
                midi_value = parameter_value_to_midi_value((self.mapped_object), max_value=(self._max_value))
                if len(self._feedback_values) > midi_value:
                    midi_value = self._feedback_values[midi_value]
                    if isinstance(midi_value, tuple):
                        midi_value = midi_value[0] + (midi_value[1] << 7)
                self.send_value(midi_value)

    def _clear_parameter_listeners(self):
        self._EncoderElement__on_parameter_name_changed.subject = None
        self._EncoderElement__on_parameter_value_changed.subject = None

    def _update_parameter_listeners(self):
        self._EncoderElement__on_parameter_name_changed.subject = self.mapped_object
        self._EncoderElement__on_parameter_value_changed.subject = self.mapped_object
        self._EncoderElement__on_parameter_name_changed()
        self._EncoderElement__on_parameter_value_changed()
        self.notify_parameter()

    def _unblock_internal_parameter_feedback(self):
        self._block_internal_parameter_feedback = False
        self._parameter_value_changed()

    @lazy_attribute
    def _unblock_internal_parameter_feedback_task(self):
        unblocking_task = self._tasks.add(
            task.sequence(
                task.wait(self._mapping_feedback_delay / 10),
                task.run(self._unblock_internal_parameter_feedback),
            ),
        )
        unblocking_task.kill()
        return unblocking_task

    @listens("name")
    def __on_parameter_name_changed(self):
        self.notify_parameter_name()

    @listens("value")
    def __on_parameter_value_changed(self):
        self.notify_parameter_value()
        self._parameter_value_changed()

    @listens("value")
    def __on_sensitivity_modifier_value(self, _):
        self._set_sensitivity(self._sensitivity_modifier.is_pressed)

    def _set_sensitivity(self, use_fine_grain):
        self._sensitivity.should_use_fine_grain(use_fine_grain)



from __future__ import annotations

from typing import Any, Callable

from Live import *
from Live.Application import *
from Live.Base import *
from Live.Browser import *
from Live.CcControlDevice import *
from Live.Chain import *
from Live.ChainMixerDevice import *
from Live.Clip import *
from Live.CompressorDevice import *
from Live.Conversions import *
from Live.Device import *
from Live.DeviceIO import *
from Live.DeviceParameter import *
from Live.DriftDevice import *
from Live.DrumChain import *
from Live.DrumPad import *
from Live.Eq8Device import *
from Live.Groove import *
from Live.GroovePool import *
from Live.HybridReverbDevice import *
from Live.Listener import *
from Live.LomObject import *
from Live.MaxDevice import *
from Live.MeldDevice import *
from Live.MidiMap import *
from Live.MixerDevice import *
from Live.PluginDevice import *
from Live.RackDevice import *
from Live.RoarDevice import *
from Live.Sample import *
from Live.Scene import *
from Live.ShifterDevice import *
from Live.SimplerDevice import *
from Live.Song import *
from Live.SpectralResonatorDevice import *
from Live.Track import *
from Live.WavetableDevice import *


class ClipSlot:
    """
    This class represents an entry in Lives Session view matrix.
    """

    def add_color_index_listener(self, listener: Callable) -> None:
        """
        add_color_index_listener( (ClipSlot)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "color_index" has changed.

            C++ signature :
                void add_color_index_listener(TPyHandle<AGroupAndClipSlotBase>,boost::python::api::object)
        """

    def add_color_listener(self, listener: Callable) -> None:
        """
        add_color_listener( (ClipSlot)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "color" has changed.

            C++ signature :
                void add_color_listener(TPyHandle<AGroupAndClipSlotBase>,boost::python::api::object)
        """

    def add_controls_other_clips_listener(self, listener: Callable) -> None:
        """
        add_controls_other_clips_listener( (ClipSlot)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "controls_other_clips" has changed.

            C++ signature :
                void add_controls_other_clips_listener(TPyHandle<AGroupAndClipSlotBase>,boost::python::api::object)
        """

    def add_has_clip_listener(self, listener: Callable) -> None:
        """
        add_has_clip_listener( (ClipSlot)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "has_clip" has changed.

            C++ signature :
                void add_has_clip_listener(TPyHandle<AGroupAndClipSlotBase>,boost::python::api::object)
        """

    def add_has_stop_button_listener(self, listener: Callable) -> None:
        """
        add_has_stop_button_listener( (ClipSlot)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "has_stop_button" has changed.

            C++ signature :
                void add_has_stop_button_listener(TPyHandle<AGroupAndClipSlotBase>,boost::python::api::object)
        """

    def add_is_triggered_listener(self, listener: Callable) -> None:
        """
        add_is_triggered_listener( (ClipSlot)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "is_triggered" has changed.

            C++ signature :
                void add_is_triggered_listener(TPyHandle<AGroupAndClipSlotBase>,boost::python::api::object)
        """

    def add_playing_status_listener(self, listener: Callable) -> None:
        """
        add_playing_status_listener( (ClipSlot)arg1, (object)arg2) -> None :
            Add a listener function or method, which will be called as soon as the
            property "playing_status" has changed.

            C++ signature :
                void add_playing_status_listener(TPyHandle<AGroupAndClipSlotBase>,boost::python::api::object)
        """

    def color_has_listener(self, listener: Callable) -> bool:
        """
        color_has_listener( (ClipSlot)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "color".

            C++ signature :
                bool color_has_listener(TPyHandle<AGroupAndClipSlotBase>,boost::python::api::object)
        """

    def color_index_has_listener(self, listener: Callable) -> bool:
        """
        color_index_has_listener( (ClipSlot)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "color_index".

            C++ signature :
                bool color_index_has_listener(TPyHandle<AGroupAndClipSlotBase>,boost::python::api::object)
        """

    def controls_other_clips_has_listener(self, listener: Callable) -> bool:
        """
        controls_other_clips_has_listener( (ClipSlot)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "controls_other_clips".

            C++ signature :
                bool controls_other_clips_has_listener(TPyHandle<AGroupAndClipSlotBase>,boost::python::api::object)
        """

    def create_clip(self, arg2: float) -> None:
        """
        create_clip( (ClipSlot)arg1, (float)arg2) -> None :
            Creates an empty clip with the given length in the slot.
            Throws an error when called on non-empty slots or slots in non-MIDI tracks.

            C++ signature :
                void create_clip(TPyHandle<AGroupAndClipSlotBase>,double)
        """

    def delete_clip(self) -> None:
        """
        delete_clip( (ClipSlot)arg1) -> None :
            Removes the clip contained in the slot.
            Raises an exception if the slot was empty.

            C++ signature :
                void delete_clip(TPyHandle<AGroupAndClipSlotBase>)
        """

    def duplicate_clip_to(self, arg2: object) -> None:
        """
        duplicate_clip_to( (ClipSlot)arg1, (ClipSlot)arg2) -> None :
            Duplicates the slot's clip to the passed in target slot.
            Overrides the target's clip if it's not empty.
            Raises an exception if the (source) slot itself is empty, or if source and
            target have different track types (audio vs. MIDI). Also raises if the source
            or target slot is in a group track (so called group slot).

            C++ signature :
                void duplicate_clip_to(TPyHandle<AGroupAndClipSlotBase>,TPyHandle<AGroupAndClipSlotBase>)
        """

    def fire(self) -> None:
        """
        fire( (ClipSlot)arg1) -> None :
            Fire a Clip if this Clipslot owns one, else trigger the stop button,
            if we have one.

            C++ signature :
                void fire(TPyHandle<AGroupAndClipSlotBase>)

        fire( (ClipSlot)self [, (float)record_length=1.7976931348623157e+308 [, (int)launch_quantization=-2147483648 [, (bool)force_legato=False]]]) -> None :
            If 'record_length' is passed, the clip will be refired after the given recording length.  Raises an error if the slot owns a clip.
            'launch_quantization' determines the quantization of global transport that is applied overriding the value in the song.
            'force_legato' will make the clip play inmediatelly. The playhead will be moved to keep the clip synchronized.

            C++ signature :
                void fire(TPyHandle<AGroupAndClipSlotBase> [,double=1.7976931348623157e+308 [,int=-2147483648 [,bool=False]]])
        """

    def has_clip_has_listener(self, listener: Callable) -> bool:
        """
        has_clip_has_listener( (ClipSlot)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "has_clip".

            C++ signature :
                bool has_clip_has_listener(TPyHandle<AGroupAndClipSlotBase>,boost::python::api::object)
        """

    def has_stop_button_has_listener(self, listener: Callable) -> bool:
        """
        has_stop_button_has_listener( (ClipSlot)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "has_stop_button".

            C++ signature :
                bool has_stop_button_has_listener(TPyHandle<AGroupAndClipSlotBase>,boost::python::api::object)
        """

    def is_triggered_has_listener(self, listener: Callable) -> bool:
        """
        is_triggered_has_listener( (ClipSlot)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "is_triggered".

            C++ signature :
                bool is_triggered_has_listener(TPyHandle<AGroupAndClipSlotBase>,boost::python::api::object)
        """

    def playing_status_has_listener(self, listener: Callable) -> bool:
        """
        playing_status_has_listener( (ClipSlot)arg1, (object)arg2) -> bool :
            Returns true, if the given listener function or method is connected
            to the property "playing_status".

            C++ signature :
                bool playing_status_has_listener(TPyHandle<AGroupAndClipSlotBase>,boost::python::api::object)
        """

    def remove_color_index_listener(self, listener: Callable) -> None:
        """
        remove_color_index_listener( (ClipSlot)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "color_index".

            C++ signature :
                void remove_color_index_listener(TPyHandle<AGroupAndClipSlotBase>,boost::python::api::object)
        """

    def remove_color_listener(self, listener: Callable) -> None:
        """
        remove_color_listener( (ClipSlot)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "color".

            C++ signature :
                void remove_color_listener(TPyHandle<AGroupAndClipSlotBase>,boost::python::api::object)
        """

    def remove_controls_other_clips_listener(self, listener: Callable) -> None:
        """
        remove_controls_other_clips_listener( (ClipSlot)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "controls_other_clips".

            C++ signature :
                void remove_controls_other_clips_listener(TPyHandle<AGroupAndClipSlotBase>,boost::python::api::object)
        """

    def remove_has_clip_listener(self, listener: Callable) -> None:
        """
        remove_has_clip_listener( (ClipSlot)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "has_clip".

            C++ signature :
                void remove_has_clip_listener(TPyHandle<AGroupAndClipSlotBase>,boost::python::api::object)
        """

    def remove_has_stop_button_listener(self, listener: Callable) -> None:
        """
        remove_has_stop_button_listener( (ClipSlot)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "has_stop_button".

            C++ signature :
                void remove_has_stop_button_listener(TPyHandle<AGroupAndClipSlotBase>,boost::python::api::object)
        """

    def remove_is_triggered_listener(self, listener: Callable) -> None:
        """
        remove_is_triggered_listener( (ClipSlot)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "is_triggered".

            C++ signature :
                void remove_is_triggered_listener(TPyHandle<AGroupAndClipSlotBase>,boost::python::api::object)
        """

    def remove_playing_status_listener(self, listener: Callable) -> None:
        """
        remove_playing_status_listener( (ClipSlot)arg1, (object)arg2) -> None :
            Remove a previously set listener function or method from
            property "playing_status".

            C++ signature :
                void remove_playing_status_listener(TPyHandle<AGroupAndClipSlotBase>,boost::python::api::object)
        """

    def set_fire_button_state(self, arg2: bool) -> None:
        """
        set_fire_button_state( (ClipSlot)arg1, (bool)arg2) -> None :
            Set the clipslot's fire button state directly. Supports all launch modes.

            C++ signature :
                void set_fire_button_state(TPyHandle<AGroupAndClipSlotBase>,bool)
        """

    def stop(self) -> None:
        """
        stop( (ClipSlot)arg1) -> None :
            Stop playing the contained Clip, if there is a Clip and its currently
            playing.

            C++ signature :
                void stop(TPyHandle<AGroupAndClipSlotBase>)
        """

    @property
    def canonical_parent(self) -> Any:
        """
        Get the canonical parent of the ClipSlot.
        """

    @property
    def clip(self) -> Clip:
        """
        Returns the Clip which this clipslots currently owns. Might be None.
        """

    @property
    def color(self) -> Any:
        """
        Returns the canonical color for the clip slot or None if it does not exist.
        """

    @property
    def color_index(self) -> Any:
        """
        Returns the canonical color index for the clip slot or None if it does not exist.
        """

    @property
    def controls_other_clips(self) -> list[Clip]:
        """
        Returns true if firing this slot will fire clips in other slots.
        Can only be true for slots in group tracks.
        """

    @property
    def has_clip(self) -> bool:
        """
        Returns true if this Clipslot owns a Clip.
        """

    @property
    def has_stop_button(self) -> bool:
        """
        Get/Set if this Clip has a stop button, which will, if fired, stop any
        other Clip that is currently playing the Track we do belong to.
        """

    @has_stop_button.setter
    def has_stop_button(self, value: bool) -> None:
        pass

    @property
    def is_group_slot(self) -> bool:
        """
        Returns whether this clip slot is a group track slot (group slot).
        """

    @property
    def is_playing(self) -> bool:
        """
        Returns whether the clip associated with the slot is playing.
        """

    @property
    def is_recording(self) -> bool:
        """
        Returns whether the clip associated with the slot is recording.
        """

    @property
    def is_triggered(self) -> bool:
        """
        Const access to the triggering state of the clip slot.
        """

    @property
    def playing_status(self) -> Any:
        """
        Const access to the playing state of the clip slot.
        Can be either stopped, playing, or recording.
        """

    @property
    def will_record_on_start(self) -> Any:
        """
        returns true if the clip slot will record on being fired.
        """


class ClipSlotPlayingState:
    """
    Class that represent an enumeration of values for ClipSlotPlayingState
    """

    stopped = 0
    started = 1
    recording = 2



# decompyle3 version 3.9.0
# Python bytecode version base 3.7.0 (3394)
# Decompiled from: Python 3.7.16 (default, Jan 17 2023, 09:28:58)
# [Clang 14.0.6 ]
# Embedded file name: output/Live/mac_universal_64_static/Release/python-bundle/MIDI Remote Scripts/ableton/v3/base/util.py
# Compiled at: 2023-11-21 10:21:18
# Size of source mod 2**32: 2731 bytes
from . import is_iterable

PITCH_NAMES = ("C", "C♯/D♭", "D", "D♯/E♭", "E", "F", "F♯/G♭", "G", "G♯/A♭", "A", "A♯/B♭", "B")


class CallableBool:
    def __init__(self, value: bool):
        self.value = value

    def __call__(self):
        return self.value

    def __eq__(self, other):
        return self.value == other

    def __bool__(self):
        return self.value

    def __int__(self):
        return int(self.value)

    def __repr__(self):
        return repr(self.value)


def get_default_ascii_translations():
    ascii_translations = {chr(i): i for i in range(32, 127)}
    ascii_translations["♯"] = 35
    return ascii_translations


DEFAULT_ASCII_TRANSLATIONS = get_default_ascii_translations()


def as_ascii(string, ascii_translations=DEFAULT_ASCII_TRANSLATIONS):
    result = []
    for char in string:
        translated_char = ascii_translations.get(char, ascii_translations["?"])
        if is_iterable(translated_char):
            result.extend(translated_char)
        else:
            result.append(translated_char)

    return result


def hex_to_rgb(hex_value):
    return ((hex_value & 16711680) >> 16, (hex_value & 65280) >> 8, hex_value & 255)


def pitch_index_to_string(index, pitch_names=PITCH_NAMES):
    return pitch_names[index % 12] + str(index // 12 - 2)



# decompyle3 version 3.9.0
# Python bytecode version base 3.7.0 (3394)
# Decompiled from: Python 3.7.16 (default, Jan 17 2023, 09:28:58)
# [Clang 14.0.6 ]
# Embedded file name: output/Live/mac_universal_64_static/Release/python-bundle/MIDI Remote Scripts/ableton/v2/control_surface/control/sysex.py
# Compiled at: 2023-11-21 10:21:18
# Size of source mod 2**32: 1044 bytes
from .control import Control, control_color


class ColorSysexControl(Control):
    class State(Control.State):
        color = control_color("DefaultButton.Disabled")

        def __init__(self, color=None, *a, **k):
            (super(ColorSysexControl.State, self).__init__)(*a, **k)
            if color is not None:
                self.color = color

        def set_control_element(self, control_element):
            super(ColorSysexControl.State, self).set_control_element(control_element)
            self._send_current_color()

        def update(self):
            super(ColorSysexControl.State, self).update()
            self._send_current_color()

        def _send_current_color(self):
            if self._control_element:
                self._control_element.set_light(self.color)

    def __init__(self, *a, **k):
        super().__init__(extra_args=a, extra_kws=k)



# decompyle3 version 3.9.0
# Python bytecode version base 3.7.0 (3394)
# Decompiled from: Python 3.7.16 (default, Jan 17 2023, 09:28:58)
# [Clang 14.0.6 ]
# Embedded file name: output/Live/mac_universal_64_static/Release/python-bundle/MIDI Remote Scripts/ableton/v3/control_surface/display/__init__.py
# Compiled at: 2023-11-21 10:21:18
# Size of source mod 2**32: 674 bytes
from .display_specification import Display, DisplaySpecification
from .notifications import DefaultNotifications, Notifications
from .renderable import Renderable
from .state import State, StateFilters
from .text import Text
from .type_decl import Event
from .util import auto_break_lines, updating_display

__all__ = (
    "DefaultNotifications",
    "Display",
    "DisplaySpecification",
    "Notifications",
    "Renderable",
    "Event",
    "State",
    "StateFilters",
    "Text",
    "auto_break_lines",
    "updating_display",
)
